@@ -91,6 +91,10 @@
    #include <botan/slh_dsa.h>
 #endif
 
+#if defined(BOTAN_HAS_CLASSICMCELIECE)
+   #include <botan/cmce.h>
+#endif
+
 namespace {
 
 #if defined(BOTAN_HAS_ECC_PUBLIC_KEY_CRYPTO)
@@ -102,6 +106,9 @@ int privkey_load_ec(std::unique_ptr<ECPrivateKey_t>& key, const Botan::BigInt& s
    if(curve_name == nullptr) {
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
+   if(!Botan::EC_Group::supports_named_group(curve_name)) {
+      return BOTAN_FFI_ERROR_NOT_IMPLEMENTED;
+   }
 
    Botan::Null_RNG null_rng;
    const auto grp = Botan::EC_Group::from_name(curve_name);
@@ -118,10 +125,14 @@ int pubkey_load_ec(std::unique_ptr<ECPublicKey_t>& key,
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
 
+   if(!Botan::EC_Group::supports_named_group(curve_name)) {
+      return BOTAN_FFI_ERROR_NOT_IMPLEMENTED;
+   }
+
    const auto group = Botan::EC_Group::from_name(curve_name);
 
    if(auto pt = Botan::EC_AffinePoint::from_bigint_xy(group, public_x, public_y)) {
-      key.reset(new ECPublicKey_t(group, pt->to_legacy_point()));
+      key.reset(new ECPublicKey_t(group, pt.value()));
       return BOTAN_FFI_SUCCESS;
    } else {
       return BOTAN_FFI_ERROR_BAD_PARAMETER;
@@ -135,9 +146,9 @@ Botan::BigInt pubkey_get_field(const Botan::Public_Key& key, std::string_view fi
    // Not currently handled by get_int_field
    if(const Botan::EC_PublicKey* ecc = dynamic_cast<const Botan::EC_PublicKey*>(&key)) {
       if(field == "public_x") {
-         return ecc->public_point().get_affine_x();
+         return Botan::BigInt::from_bytes(ecc->_public_ec_point().x_bytes());
       } else if(field == "public_y") {
-         return ecc->public_point().get_affine_y();
+         return Botan::BigInt::from_bytes(ecc->_public_ec_point().y_bytes());
       }
    }
 #endif
@@ -154,9 +165,9 @@ Botan::BigInt privkey_get_field(const Botan::Private_Key& key, std::string_view
    // Not currently handled by get_int_field
    if(const Botan::EC_PublicKey* ecc = dynamic_cast<const Botan::EC_PublicKey*>(&key)) {
       if(field == "public_x") {
-         return ecc->public_point().get_affine_x();
+         return Botan::BigInt::from_bytes(ecc->_public_ec_point().x_bytes());
       } else if(field == "public_y") {
-         return ecc->public_point().get_affine_y();
+         return Botan::BigInt::from_bytes(ecc->_public_ec_point().y_bytes());
       }
    }
 #endif
@@ -208,12 +219,14 @@ int botan_privkey_create_rsa(botan_privkey_t* key_obj, botan_rng_t rng_obj, size
 
 int botan_privkey_load_rsa(botan_privkey_t* key, botan_mp_t rsa_p, botan_mp_t rsa_q, botan_mp_t rsa_e) {
 #if defined(BOTAN_HAS_RSA)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
 
    return ffi_guard_thunk(__func__, [=]() -> int {
       auto rsa = std::make_unique<Botan::RSA_PrivateKey>(safe_get(rsa_p), safe_get(rsa_q), safe_get(rsa_e));
-      *key = new botan_privkey_struct(std::move(rsa));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(rsa));
    });
 #else
    BOTAN_UNUSED(key, rsa_p, rsa_q, rsa_e);
@@ -223,14 +236,15 @@ int botan_privkey_load_rsa(botan_privkey_t* key, botan_mp_t rsa_p, botan_mp_t rs
 
 int botan_privkey_load_rsa_pkcs1(botan_privkey_t* key, const uint8_t bits[], size_t len) {
 #if defined(BOTAN_HAS_RSA)
+   if(key == nullptr || bits == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
 
-   Botan::secure_vector<uint8_t> src(bits, bits + len);
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::AlgorithmIdentifier alg_id("RSA", Botan::AlgorithmIdentifier::USE_NULL_PARAM);
-      auto rsa = std::make_unique<Botan::RSA_PrivateKey>(alg_id, src);
-      *key = new botan_privkey_struct(std::move(rsa));
-      return BOTAN_FFI_SUCCESS;
+      auto rsa = std::make_unique<Botan::RSA_PrivateKey>(alg_id, std::span{bits, len});
+      return ffi_new_object(key, std::move(rsa));
    });
 #else
    BOTAN_UNUSED(key, bits, len);
@@ -240,11 +254,13 @@ int botan_privkey_load_rsa_pkcs1(botan_privkey_t* key, const uint8_t bits[], siz
 
 int botan_pubkey_load_rsa(botan_pubkey_t* key, botan_mp_t n, botan_mp_t e) {
 #if defined(BOTAN_HAS_RSA)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       auto rsa = std::make_unique<Botan::RSA_PublicKey>(safe_get(n), safe_get(e));
-      *key = new botan_pubkey_struct(std::move(rsa));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(rsa));
    });
 #else
    BOTAN_UNUSED(key, n, e);
@@ -284,12 +300,16 @@ int botan_privkey_rsa_get_privkey(botan_privkey_t rsa_key, uint8_t out[], size_t
 #if defined(BOTAN_HAS_RSA)
    return BOTAN_FFI_VISIT(rsa_key, [=](const auto& k) -> int {
       if(const Botan::RSA_PrivateKey* rsa = dynamic_cast<const Botan::RSA_PrivateKey*>(&k)) {
-         if(flags == BOTAN_PRIVKEY_EXPORT_FLAG_DER)
+         if(flags == BOTAN_PRIVKEY_EXPORT_FLAG_DER) {
             return write_vec_output(out, out_len, rsa->private_key_bits());
-         else if(flags == BOTAN_PRIVKEY_EXPORT_FLAG_PEM)
-            return write_str_output(out, out_len, Botan::PEM_Code::encode(rsa->private_key_bits(), "RSA PRIVATE KEY"));
-         else
+         } else if(flags == BOTAN_PRIVKEY_EXPORT_FLAG_PEM) {
+            // TODO define new generic functions for this
+            return write_str_output(reinterpret_cast<char*>(out),
+                                    out_len,
+                                    Botan::PEM_Code::encode(rsa->private_key_bits(), "RSA PRIVATE KEY"));
+         } else {
             return BOTAN_FFI_ERROR_BAD_FLAG;
+         }
       } else {
          return BOTAN_FFI_ERROR_BAD_PARAMETER;
       }
@@ -308,7 +328,7 @@ int botan_privkey_create_dsa(botan_privkey_t* key, botan_rng_t rng_obj, size_t p
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
 
-   if((pbits % 64) || (qbits % 8) || (pbits < 1024) || (pbits > 3072) || (qbits < 160) || (qbits > 256)) {
+   if((pbits % 64 != 0) || (qbits % 8 != 0) || (pbits < 1024) || (pbits > 3072) || (qbits < 160) || (qbits > 256)) {
       return BOTAN_FFI_ERROR_BAD_PARAMETER;
    }
 
@@ -316,8 +336,7 @@ int botan_privkey_create_dsa(botan_privkey_t* key, botan_rng_t rng_obj, size_t p
       Botan::RandomNumberGenerator& rng = safe_get(rng_obj);
       Botan::DL_Group group(rng, Botan::DL_Group::Prime_Subgroup, pbits, qbits);
       auto dsa = std::make_unique<Botan::DSA_PrivateKey>(rng, group);
-      *key = new botan_privkey_struct(std::move(dsa));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(dsa));
    });
 #else
    BOTAN_UNUSED(key, rng_obj, pbits, qbits);
@@ -327,13 +346,15 @@ int botan_privkey_create_dsa(botan_privkey_t* key, botan_rng_t rng_obj, size_t p
 
 int botan_privkey_load_dsa(botan_privkey_t* key, botan_mp_t p, botan_mp_t q, botan_mp_t g, botan_mp_t x) {
 #if defined(BOTAN_HAS_DSA)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
 
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(q), safe_get(g));
       auto dsa = std::make_unique<Botan::DSA_PrivateKey>(group, safe_get(x));
-      *key = new botan_privkey_struct(std::move(dsa));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(dsa));
    });
 #else
    BOTAN_UNUSED(key, p, q, g, x);
@@ -343,13 +364,15 @@ int botan_privkey_load_dsa(botan_privkey_t* key, botan_mp_t p, botan_mp_t q, bot
 
 int botan_pubkey_load_dsa(botan_pubkey_t* key, botan_mp_t p, botan_mp_t q, botan_mp_t g, botan_mp_t y) {
 #if defined(BOTAN_HAS_DSA)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
 
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(q), safe_get(g));
       auto dsa = std::make_unique<Botan::DSA_PublicKey>(group, safe_get(y));
-      *key = new botan_pubkey_struct(std::move(dsa));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(dsa));
    });
 #else
    BOTAN_UNUSED(key, p, q, g, y);
@@ -401,17 +424,24 @@ int botan_pubkey_ecc_key_used_explicit_encoding(botan_pubkey_t key) {
 #endif
 }
 
+// NOLINTBEGIN(misc-misplaced-const)
+
 int botan_pubkey_load_ecdsa(botan_pubkey_t* key,
                             const botan_mp_t public_x,
                             const botan_mp_t public_y,
                             const char* curve_name) {
 #if defined(BOTAN_HAS_ECDSA)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
+
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::ECDSA_PublicKey> p_key;
 
       int rc = pubkey_load_ec(p_key, safe_get(public_x), safe_get(public_y), curve_name);
       if(rc == BOTAN_FFI_SUCCESS) {
-         *key = new botan_pubkey_struct(std::move(p_key));
+         ffi_new_object(key, std::move(p_key));
       }
 
       return rc;
@@ -424,11 +454,16 @@ int botan_pubkey_load_ecdsa(botan_pubkey_t* key,
 
 int botan_privkey_load_ecdsa(botan_privkey_t* key, const botan_mp_t scalar, const char* curve_name) {
 #if defined(BOTAN_HAS_ECDSA)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
+
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::ECDSA_PrivateKey> p_key;
       int rc = privkey_load_ec(p_key, safe_get(scalar), curve_name);
       if(rc == BOTAN_FFI_SUCCESS) {
-         *key = new botan_privkey_struct(std::move(p_key));
+         ffi_new_object(key, std::move(p_key));
       }
       return rc;
    });
@@ -441,12 +476,12 @@ int botan_privkey_load_ecdsa(botan_privkey_t* key, const botan_mp_t scalar, cons
 /* ElGamal specific operations */
 int botan_privkey_create_elgamal(botan_privkey_t* key, botan_rng_t rng_obj, size_t pbits, size_t qbits) {
 #if defined(BOTAN_HAS_ELGAMAL)
-
-   if((rng_obj == nullptr) || (key == nullptr)) {
+   if(key == nullptr || rng_obj == nullptr) {
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
+   *key = nullptr;
 
-   if((pbits < 1024) || (qbits < 160)) {
+   if(pbits < 1024 || qbits < 160) {
       return BOTAN_FFI_ERROR_BAD_PARAMETER;
    }
 
@@ -457,8 +492,7 @@ int botan_privkey_create_elgamal(botan_privkey_t* key, botan_rng_t rng_obj, size
       Botan::RandomNumberGenerator& rng = safe_get(rng_obj);
       Botan::DL_Group group(rng, prime_type, pbits, qbits);
       auto elg = std::make_unique<Botan::ElGamal_PrivateKey>(rng, group);
-      *key = new botan_privkey_struct(std::move(elg));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(elg));
    });
 #else
    BOTAN_UNUSED(key, rng_obj, pbits, qbits);
@@ -468,12 +502,14 @@ int botan_privkey_create_elgamal(botan_privkey_t* key, botan_rng_t rng_obj, size
 
 int botan_pubkey_load_elgamal(botan_pubkey_t* key, botan_mp_t p, botan_mp_t g, botan_mp_t y) {
 #if defined(BOTAN_HAS_ELGAMAL)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(g));
       auto elg = std::make_unique<Botan::ElGamal_PublicKey>(group, safe_get(y));
-      *key = new botan_pubkey_struct(std::move(elg));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(elg));
    });
 #else
    BOTAN_UNUSED(key, p, g, y);
@@ -483,12 +519,14 @@ int botan_pubkey_load_elgamal(botan_pubkey_t* key, botan_mp_t p, botan_mp_t g, b
 
 int botan_privkey_load_elgamal(botan_privkey_t* key, botan_mp_t p, botan_mp_t g, botan_mp_t x) {
 #if defined(BOTAN_HAS_ELGAMAL)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(g));
       auto elg = std::make_unique<Botan::ElGamal_PrivateKey>(group, safe_get(x));
-      *key = new botan_privkey_struct(std::move(elg));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(elg));
    });
 #else
    BOTAN_UNUSED(key, p, g, x);
@@ -504,12 +542,14 @@ int botan_privkey_create_dh(botan_privkey_t* key_obj, botan_rng_t rng_obj, const
 
 int botan_privkey_load_dh(botan_privkey_t* key, botan_mp_t p, botan_mp_t g, botan_mp_t x) {
 #if defined(BOTAN_HAS_DIFFIE_HELLMAN)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(g));
       auto dh = std::make_unique<Botan::DH_PrivateKey>(group, safe_get(x));
-      *key = new botan_privkey_struct(std::move(dh));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(dh));
    });
 #else
    BOTAN_UNUSED(key, p, g, x);
@@ -519,12 +559,14 @@ int botan_privkey_load_dh(botan_privkey_t* key, botan_mp_t p, botan_mp_t g, bota
 
 int botan_pubkey_load_dh(botan_pubkey_t* key, botan_mp_t p, botan_mp_t g, botan_mp_t y) {
 #if defined(BOTAN_HAS_DIFFIE_HELLMAN)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       Botan::DL_Group group(safe_get(p), safe_get(g));
       auto dh = std::make_unique<Botan::DH_PublicKey>(group, safe_get(y));
-      *key = new botan_pubkey_struct(std::move(dh));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(dh));
    });
 #else
    BOTAN_UNUSED(key, p, g, y);
@@ -535,17 +577,18 @@ int botan_pubkey_load_dh(botan_pubkey_t* key, botan_mp_t p, botan_mp_t g, botan_
 /* ECDH + x25519/x448 specific operations */
 
 int botan_privkey_create_ecdh(botan_privkey_t* key_obj, botan_rng_t rng_obj, const char* param_str) {
-   if(param_str == nullptr) {
+   if(key_obj == nullptr || param_str == nullptr) {
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
+   *key_obj = nullptr;
 
    const std::string params(param_str);
 
-   if(params == "x25519" || params == "curve25519") {
+   if(params == "X25519" || params == "x25519" || params == "curve25519") {
       return botan_privkey_create(key_obj, "X25519", "", rng_obj);
    }
 
-   if(params == "x448") {
+   if(params == "X448" || params == "x448") {
       return botan_privkey_create(key_obj, "X448", "", rng_obj);
    }
 
@@ -557,12 +600,16 @@ int botan_pubkey_load_ecdh(botan_pubkey_t* key,
                            const botan_mp_t public_y,
                            const char* curve_name) {
 #if defined(BOTAN_HAS_ECDH)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::ECDH_PublicKey> p_key;
       int rc = pubkey_load_ec(p_key, safe_get(public_x), safe_get(public_y), curve_name);
 
       if(rc == BOTAN_FFI_SUCCESS) {
-         *key = new botan_pubkey_struct(std::move(p_key));
+         ffi_new_object(key, std::move(p_key));
       }
       return rc;
    });
@@ -574,11 +621,15 @@ int botan_pubkey_load_ecdh(botan_pubkey_t* key,
 
 int botan_privkey_load_ecdh(botan_privkey_t* key, const botan_mp_t scalar, const char* curve_name) {
 #if defined(BOTAN_HAS_ECDH)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::ECDH_PrivateKey> p_key;
       int rc = privkey_load_ec(p_key, safe_get(scalar), curve_name);
       if(rc == BOTAN_FFI_SUCCESS) {
-         *key = new botan_privkey_struct(std::move(p_key));
+         ffi_new_object(key, std::move(p_key));
       }
       return rc;
    });
@@ -592,10 +643,7 @@ int botan_privkey_load_ecdh(botan_privkey_t* key, const botan_mp_t scalar, const
 
 int botan_pubkey_sm2_compute_za(
    uint8_t out[], size_t* out_len, const char* ident, const char* hash_algo, const botan_pubkey_t key) {
-   if(out == nullptr || out_len == nullptr) {
-      return BOTAN_FFI_ERROR_NULL_POINTER;
-   }
-   if(ident == nullptr || hash_algo == nullptr || key == nullptr) {
+   if(out == nullptr || out_len == nullptr || ident == nullptr || hash_algo == nullptr || key == nullptr) {
       return BOTAN_FFI_ERROR_NULL_POINTER;
    }
 
@@ -615,7 +663,9 @@ int botan_pubkey_sm2_compute_za(
       const std::string ident_str(ident);
       std::unique_ptr<Botan::HashFunction> hash = Botan::HashFunction::create_or_throw(hash_algo);
 
-      const std::vector<uint8_t> za = Botan::sm2_compute_za(*hash, ident_str, ec_key->domain(), ec_key->public_point());
+      const auto& pt = ec_key->_public_ec_point();
+
+      const auto za = Botan::sm2_compute_za(*hash, ident_str, ec_key->domain(), pt);
 
       return write_vec_output(out, out_len, za);
    });
@@ -629,13 +679,18 @@ int botan_pubkey_load_sm2(botan_pubkey_t* key,
                           const botan_mp_t public_y,
                           const char* curve_name) {
 #if defined(BOTAN_HAS_SM2)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
+
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::SM2_PublicKey> p_key;
-      if(!pubkey_load_ec(p_key, safe_get(public_x), safe_get(public_y), curve_name)) {
-         *key = new botan_pubkey_struct(std::move(p_key));
-         return BOTAN_FFI_SUCCESS;
+      if(pubkey_load_ec(p_key, safe_get(public_x), safe_get(public_y), curve_name) == 0) {
+         return ffi_new_object(key, std::move(p_key));
+      } else {
+         return BOTAN_FFI_ERROR_UNKNOWN_ERROR;
       }
-      return BOTAN_FFI_ERROR_UNKNOWN_ERROR;
    });
 #else
    BOTAN_UNUSED(key, public_x, public_y, curve_name);
@@ -645,12 +700,17 @@ int botan_pubkey_load_sm2(botan_pubkey_t* key,
 
 int botan_privkey_load_sm2(botan_privkey_t* key, const botan_mp_t scalar, const char* curve_name) {
 #if defined(BOTAN_HAS_SM2)
+   if(key == nullptr || curve_name == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+   *key = nullptr;
+
    return ffi_guard_thunk(__func__, [=]() -> int {
       std::unique_ptr<Botan::SM2_PrivateKey> p_key;
       int rc = privkey_load_ec(p_key, safe_get(scalar), curve_name);
 
       if(rc == BOTAN_FFI_SUCCESS) {
-         *key = new botan_privkey_struct(std::move(p_key));
+         ffi_new_object(key, std::move(p_key));
       }
       return rc;
    });
@@ -675,12 +735,14 @@ int botan_privkey_load_sm2_enc(botan_privkey_t* key, const botan_mp_t scalar, co
 
 int botan_privkey_load_ed25519(botan_privkey_t* key, const uint8_t privkey[32]) {
 #if defined(BOTAN_HAS_ED25519)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
-      const Botan::secure_vector<uint8_t> privkey_vec(privkey, privkey + 32);
-      auto ed25519 = std::make_unique<Botan::Ed25519_PrivateKey>(privkey_vec);
-      *key = new botan_privkey_struct(std::move(ed25519));
-      return BOTAN_FFI_SUCCESS;
+      auto ed25519 =
+         std::make_unique<Botan::Ed25519_PrivateKey>(Botan::Ed25519_PrivateKey::from_seed(std::span{privkey, 32}));
+      return ffi_new_object(key, std::move(ed25519));
    });
 #else
    BOTAN_UNUSED(key, privkey);
@@ -690,12 +752,14 @@ int botan_privkey_load_ed25519(botan_privkey_t* key, const uint8_t privkey[32])
 
 int botan_pubkey_load_ed25519(botan_pubkey_t* key, const uint8_t pubkey[32]) {
 #if defined(BOTAN_HAS_ED25519)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       const std::vector<uint8_t> pubkey_vec(pubkey, pubkey + 32);
       auto ed25519 = std::make_unique<Botan::Ed25519_PublicKey>(pubkey_vec);
-      *key = new botan_pubkey_struct(std::move(ed25519));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(ed25519));
    });
 #else
    BOTAN_UNUSED(key, pubkey);
@@ -708,8 +772,9 @@ int botan_privkey_ed25519_get_privkey(botan_privkey_t key, uint8_t output[64]) {
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto ed = dynamic_cast<const Botan::Ed25519_PrivateKey*>(&k)) {
          const auto ed_key = ed->raw_private_key_bits();
-         if(ed_key.size() != 64)
+         if(ed_key.size() != 64) {
             return BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE;
+         }
          Botan::copy_mem(output, ed_key.data(), ed_key.size());
          return BOTAN_FFI_SUCCESS;
       } else {
@@ -727,8 +792,9 @@ int botan_pubkey_ed25519_get_pubkey(botan_pubkey_t key, uint8_t output[32]) {
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto ed = dynamic_cast<const Botan::Ed25519_PublicKey*>(&k)) {
          const std::vector<uint8_t>& ed_key = ed->get_public_key();
-         if(ed_key.size() != 32)
+         if(ed_key.size() != 32) {
             return BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE;
+         }
          Botan::copy_mem(output, ed_key.data(), ed_key.size());
          return BOTAN_FFI_SUCCESS;
       } else {
@@ -745,11 +811,13 @@ int botan_pubkey_ed25519_get_pubkey(botan_pubkey_t key, uint8_t output[32]) {
 
 int botan_privkey_load_ed448(botan_privkey_t* key, const uint8_t privkey[57]) {
 #if defined(BOTAN_HAS_ED448)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       auto ed448 = std::make_unique<Botan::Ed448_PrivateKey>(std::span(privkey, 57));
-      *key = new botan_privkey_struct(std::move(ed448));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(ed448));
    });
 #else
    BOTAN_UNUSED(key, privkey);
@@ -759,11 +827,13 @@ int botan_privkey_load_ed448(botan_privkey_t* key, const uint8_t privkey[57]) {
 
 int botan_pubkey_load_ed448(botan_pubkey_t* key, const uint8_t pubkey[57]) {
 #if defined(BOTAN_HAS_ED448)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
       auto ed448 = std::make_unique<Botan::Ed448_PublicKey>(std::span(pubkey, 57));
-      *key = new botan_pubkey_struct(std::move(ed448));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(ed448));
    });
 #else
    BOTAN_UNUSED(key, pubkey);
@@ -809,12 +879,13 @@ int botan_pubkey_ed448_get_pubkey(botan_pubkey_t key, uint8_t output[57]) {
 
 int botan_privkey_load_x25519(botan_privkey_t* key, const uint8_t privkey[32]) {
 #if defined(BOTAN_HAS_X25519)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
-      const Botan::secure_vector<uint8_t> privkey_vec(privkey, privkey + 32);
-      auto x25519 = std::make_unique<Botan::X25519_PrivateKey>(privkey_vec);
-      *key = new botan_privkey_struct(std::move(x25519));
-      return BOTAN_FFI_SUCCESS;
+      auto x25519 = std::make_unique<Botan::X25519_PrivateKey>(std::span{privkey, 32});
+      return ffi_new_object(key, std::move(x25519));
    });
 #else
    BOTAN_UNUSED(key, privkey);
@@ -824,12 +895,13 @@ int botan_privkey_load_x25519(botan_privkey_t* key, const uint8_t privkey[32]) {
 
 int botan_pubkey_load_x25519(botan_pubkey_t* key, const uint8_t pubkey[32]) {
 #if defined(BOTAN_HAS_X25519)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
-      const std::vector<uint8_t> pubkey_vec(pubkey, pubkey + 32);
-      auto x25519 = std::make_unique<Botan::X25519_PublicKey>(pubkey_vec);
-      *key = new botan_pubkey_struct(std::move(x25519));
-      return BOTAN_FFI_SUCCESS;
+      auto x25519 = std::make_unique<Botan::X25519_PublicKey>(std::span{pubkey, 32});
+      return ffi_new_object(key, std::move(x25519));
    });
 #else
    BOTAN_UNUSED(key, pubkey);
@@ -842,8 +914,9 @@ int botan_privkey_x25519_get_privkey(botan_privkey_t key, uint8_t output[32]) {
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto x25519 = dynamic_cast<const Botan::X25519_PrivateKey*>(&k)) {
          const auto x25519_key = x25519->raw_private_key_bits();
-         if(x25519_key.size() != 32)
+         if(x25519_key.size() != 32) {
             return BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE;
+         }
          Botan::copy_mem(output, x25519_key.data(), x25519_key.size());
          return BOTAN_FFI_SUCCESS;
       } else {
@@ -860,10 +933,7 @@ int botan_pubkey_x25519_get_pubkey(botan_pubkey_t key, uint8_t output[32]) {
 #if defined(BOTAN_HAS_X25519)
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto x25519 = dynamic_cast<const Botan::X25519_PublicKey*>(&k)) {
-         const std::vector<uint8_t>& x25519_key = x25519->public_value();
-         if(x25519_key.size() != 32)
-            return BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE;
-         Botan::copy_mem(output, x25519_key.data(), x25519_key.size());
+         Botan::copy_mem(std::span{output, 32}, x25519->raw_public_key_bits());
          return BOTAN_FFI_SUCCESS;
       } else {
          return BOTAN_FFI_ERROR_BAD_PARAMETER;
@@ -879,11 +949,13 @@ int botan_pubkey_x25519_get_pubkey(botan_pubkey_t key, uint8_t output[32]) {
 
 int botan_privkey_load_x448(botan_privkey_t* key, const uint8_t privkey[56]) {
 #if defined(BOTAN_HAS_X448)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
-      auto x448 = std::make_unique<Botan::X448_PrivateKey>(std::span(privkey, 56));
-      *key = new botan_privkey_struct(std::move(x448));
-      return BOTAN_FFI_SUCCESS;
+      auto x448 = std::make_unique<Botan::X448_PrivateKey>(std::span{privkey, 56});
+      return ffi_new_object(key, std::move(x448));
    });
 #else
    BOTAN_UNUSED(key, privkey);
@@ -893,11 +965,13 @@ int botan_privkey_load_x448(botan_privkey_t* key, const uint8_t privkey[56]) {
 
 int botan_pubkey_load_x448(botan_pubkey_t* key, const uint8_t pubkey[56]) {
 #if defined(BOTAN_HAS_X448)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
    return ffi_guard_thunk(__func__, [=]() -> int {
-      auto x448 = std::make_unique<Botan::X448_PublicKey>(std::span(pubkey, 56));
-      *key = new botan_pubkey_struct(std::move(x448));
-      return BOTAN_FFI_SUCCESS;
+      auto x448 = std::make_unique<Botan::X448_PublicKey>(std::span{pubkey, 56});
+      return ffi_new_object(key, std::move(x448));
    });
 #else
    BOTAN_UNUSED(key, pubkey);
@@ -910,7 +984,7 @@ int botan_privkey_x448_get_privkey(botan_privkey_t key, uint8_t output[56]) {
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto x448 = dynamic_cast<const Botan::X448_PrivateKey*>(&k)) {
          const auto x448_key = x448->raw_private_key_bits();
-         Botan::copy_mem(std::span(output, 56), x448_key);
+         Botan::copy_mem(std::span{output, 56}, x448_key);
          return BOTAN_FFI_SUCCESS;
       } else {
          return BOTAN_FFI_ERROR_BAD_PARAMETER;
@@ -926,8 +1000,7 @@ int botan_pubkey_x448_get_pubkey(botan_pubkey_t key, uint8_t output[56]) {
 #if defined(BOTAN_HAS_X448)
    return BOTAN_FFI_VISIT(key, [=](const auto& k) {
       if(auto x448 = dynamic_cast<const Botan::X448_PublicKey*>(&k)) {
-         const std::vector<uint8_t>& x448_key = x448->public_value();
-         Botan::copy_mem(std::span(output, 56), x448_key);
+         Botan::copy_mem(std::span{output, 56}, x448->raw_public_key_bits());
          return BOTAN_FFI_SUCCESS;
       } else {
          return BOTAN_FFI_ERROR_BAD_PARAMETER;
@@ -945,32 +1018,30 @@ int botan_pubkey_x448_get_pubkey(botan_pubkey_t key, uint8_t output[56]) {
 
 int botan_privkey_load_kyber(botan_privkey_t* key, const uint8_t privkey[], size_t key_len) {
 #if defined(BOTAN_HAS_KYBER)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
-   switch(key_len) {
-      case 1632:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const Botan::secure_vector<uint8_t> privkey_vec(privkey, privkey + 1632);
-            auto kyber512 = std::make_unique<Botan::Kyber_PrivateKey>(privkey_vec, Botan::KyberMode::Kyber512_R3);
-            *key = new botan_privkey_struct(std::move(kyber512));
-            return BOTAN_FFI_SUCCESS;
-         });
-      case 2400:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const Botan::secure_vector<uint8_t> privkey_vec(privkey, privkey + 2400);
-            auto kyber768 = std::make_unique<Botan::Kyber_PrivateKey>(privkey_vec, Botan::KyberMode::Kyber768_R3);
-            *key = new botan_privkey_struct(std::move(kyber768));
-            return BOTAN_FFI_SUCCESS;
-         });
-      case 3168:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const Botan::secure_vector<uint8_t> privkey_vec(privkey, privkey + 3168);
-            auto kyber1024 = std::make_unique<Botan::Kyber_PrivateKey>(privkey_vec, Botan::KyberMode::Kyber1024_R3);
-            *key = new botan_privkey_struct(std::move(kyber1024));
-            return BOTAN_FFI_SUCCESS;
-         });
-      default:
-         BOTAN_UNUSED(key, privkey, key_len);
-         return BOTAN_FFI_ERROR_BAD_PARAMETER;
+
+   const auto mode = [](size_t len) -> std::optional<Botan::KyberMode> {
+      if(len == 1632) {
+         return Botan::KyberMode::Kyber512_R3;
+      } else if(len == 2400) {
+         return Botan::KyberMode::Kyber768_R3;
+      } else if(len == 3168) {
+         return Botan::KyberMode::Kyber1024_R3;
+      } else {
+         return {};
+      }
+   }(key_len);
+
+   if(mode.has_value()) {
+      return ffi_guard_thunk(__func__, [=]() -> int {
+         auto kyber = std::make_unique<Botan::Kyber_PrivateKey>(std::span{privkey, key_len}, *mode);
+         return ffi_new_object(key, std::move(kyber));
+      });
+   } else {
+      return BOTAN_FFI_ERROR_BAD_PARAMETER;
    }
 #else
    BOTAN_UNUSED(key, key_len, privkey);
@@ -980,32 +1051,28 @@ int botan_privkey_load_kyber(botan_privkey_t* key, const uint8_t privkey[], size
 
 int botan_pubkey_load_kyber(botan_pubkey_t* key, const uint8_t pubkey[], size_t key_len) {
 #if defined(BOTAN_HAS_KYBER)
+   if(key == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
    *key = nullptr;
-   switch(key_len) {
-      case 800:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const std::vector<uint8_t> pubkey_vec(pubkey, pubkey + 800);
-            auto kyber512 = std::make_unique<Botan::Kyber_PublicKey>(pubkey_vec, Botan::KyberMode::Kyber512_R3);
-            *key = new botan_pubkey_struct(std::move(kyber512));
-            return BOTAN_FFI_SUCCESS;
-         });
-      case 1184:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const std::vector<uint8_t> pubkey_vec(pubkey, pubkey + 1184);
-            auto kyber768 = std::make_unique<Botan::Kyber_PublicKey>(pubkey_vec, Botan::KyberMode::Kyber768_R3);
-            *key = new botan_pubkey_struct(std::move(kyber768));
-            return BOTAN_FFI_SUCCESS;
-         });
-      case 1568:
-         return ffi_guard_thunk(__func__, [=]() -> int {
-            const std::vector<uint8_t> pubkey_vec(pubkey, pubkey + 1568);
-            auto kyber1024 = std::make_unique<Botan::Kyber_PublicKey>(pubkey_vec, Botan::KyberMode::Kyber1024_R3);
-            *key = new botan_pubkey_struct(std::move(kyber1024));
-            return BOTAN_FFI_SUCCESS;
-         });
-      default:
-         BOTAN_UNUSED(key, pubkey, key_len);
-         return BOTAN_FFI_ERROR_BAD_PARAMETER;
+
+   const auto mode = [](size_t len) -> std::optional<Botan::KyberMode> {
+      if(len == 800) {
+         return Botan::KyberMode::Kyber512_R3;
+      } else if(len == 1184) {
+         return Botan::KyberMode::Kyber768_R3;
+      } else if(len == 1568) {
+         return Botan::KyberMode::Kyber1024_R3;
+      } else {
+         return {};
+      }
+   }(key_len);
+
+   if(mode.has_value()) {
+      auto kyber = std::make_unique<Botan::Kyber_PublicKey>(std::span{pubkey, key_len}, *mode);
+      return ffi_new_object(key, std::move(kyber));
+   } else {
+      return BOTAN_FFI_ERROR_BAD_PARAMETER;
    }
 #else
    BOTAN_UNUSED(key, pubkey, key_len);
@@ -1062,8 +1129,7 @@ int botan_privkey_load_ml_kem(botan_privkey_t* key, const uint8_t privkey[], siz
       }
 
       auto mlkem_key = std::make_unique<Botan::ML_KEM_PrivateKey>(std::span{privkey, key_len}, mode);
-      *key = new botan_privkey_struct(std::move(mlkem_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(mlkem_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, privkey, mlkem_mode);
@@ -1086,8 +1152,7 @@ int botan_pubkey_load_ml_kem(botan_pubkey_t* key, const uint8_t pubkey[], size_t
       }
 
       auto mlkem_key = std::make_unique<Botan::ML_KEM_PublicKey>(std::span{pubkey, key_len}, mode.mode());
-      *key = new botan_pubkey_struct(std::move(mlkem_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(mlkem_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, pubkey, mlkem_mode);
@@ -1114,8 +1179,7 @@ int botan_privkey_load_ml_dsa(botan_privkey_t* key, const uint8_t privkey[], siz
       }
 
       auto mldsa_key = std::make_unique<Botan::ML_DSA_PrivateKey>(std::span{privkey, key_len}, mode);
-      *key = new botan_privkey_struct(std::move(mldsa_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(mldsa_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, privkey, mldsa_mode);
@@ -1138,8 +1202,7 @@ int botan_pubkey_load_ml_dsa(botan_pubkey_t* key, const uint8_t pubkey[], size_t
       }
 
       auto mldsa_key = std::make_unique<Botan::ML_DSA_PublicKey>(std::span{pubkey, key_len}, mode);
-      *key = new botan_pubkey_struct(std::move(mldsa_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(mldsa_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, pubkey, mldsa_mode);
@@ -1166,8 +1229,7 @@ int botan_privkey_load_slh_dsa(botan_privkey_t* key, const uint8_t privkey[], si
       }
 
       auto slhdsa_key = std::make_unique<Botan::SLH_DSA_PrivateKey>(std::span{privkey, key_len}, mode);
-      *key = new botan_privkey_struct(std::move(slhdsa_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(slhdsa_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, privkey, slhdsa_mode);
@@ -1190,8 +1252,7 @@ int botan_pubkey_load_slh_dsa(botan_pubkey_t* key, const uint8_t pubkey[], size_
       }
 
       auto mldsa_key = std::make_unique<Botan::SLH_DSA_PublicKey>(std::span{pubkey, key_len}, mode);
-      *key = new botan_pubkey_struct(std::move(mldsa_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(mldsa_key));
    });
 #else
    BOTAN_UNUSED(key, key_len, pubkey, slhdsa_mode);
@@ -1214,8 +1275,7 @@ int botan_privkey_load_frodokem(botan_privkey_t* key, const uint8_t privkey[], s
    return ffi_guard_thunk(__func__, [=]() -> int {
       const auto mode = Botan::FrodoKEMMode(frodo_mode);
       auto frodo_key = std::make_unique<Botan::FrodoKEM_PrivateKey>(std::span{privkey, key_len}, mode);
-      *key = new botan_privkey_struct(std::move(frodo_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(frodo_key));
    });
 #else
    BOTAN_UNUSED(key, privkey, key_len, frodo_mode);
@@ -1234,8 +1294,7 @@ int botan_pubkey_load_frodokem(botan_pubkey_t* key, const uint8_t pubkey[], size
    return ffi_guard_thunk(__func__, [=]() -> int {
       const auto mode = Botan::FrodoKEMMode(frodo_mode);
       auto frodo_key = std::make_unique<Botan::FrodoKEM_PublicKey>(std::span{pubkey, key_len}, mode);
-      *key = new botan_pubkey_struct(std::move(frodo_key));
-      return BOTAN_FFI_SUCCESS;
+      return ffi_new_object(key, std::move(frodo_key));
    });
 #else
    BOTAN_UNUSED(key, pubkey, key_len, frodo_mode);
@@ -1243,11 +1302,59 @@ int botan_pubkey_load_frodokem(botan_pubkey_t* key, const uint8_t pubkey[], size
 #endif
 }
 
+/*
+* Algorithm specific key operations : Classic McEliece
+*/
+
+int botan_privkey_load_classic_mceliece(botan_privkey_t* key,
+                                        const uint8_t privkey[],
+                                        size_t key_len,
+                                        const char* cmce_mode) {
+#if defined(BOTAN_HAS_CLASSICMCELIECE)
+   if(key == nullptr || privkey == nullptr || cmce_mode == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+
+   *key = nullptr;
+
+   return ffi_guard_thunk(__func__, [=]() -> int {
+      const auto mode = Botan::Classic_McEliece_Parameter_Set::from_string(cmce_mode);
+      auto cmce_key = std::make_unique<Botan::Classic_McEliece_PrivateKey>(std::span{privkey, key_len}, mode);
+      return ffi_new_object(key, std::move(cmce_key));
+   });
+#else
+   BOTAN_UNUSED(key, privkey, key_len, cmce_mode);
+   return BOTAN_FFI_ERROR_NOT_IMPLEMENTED;
+#endif
+}
+
+int botan_pubkey_load_classic_mceliece(botan_pubkey_t* key,
+                                       const uint8_t pubkey[],
+                                       size_t key_len,
+                                       const char* cmce_mode) {
+#if defined(BOTAN_HAS_CLASSICMCELIECE)
+   if(key == nullptr || pubkey == nullptr || cmce_mode == nullptr) {
+      return BOTAN_FFI_ERROR_NULL_POINTER;
+   }
+
+   *key = nullptr;
+
+   return ffi_guard_thunk(__func__, [=]() -> int {
+      const auto mode = Botan::Classic_McEliece_Parameter_Set::from_string(cmce_mode);
+      auto cmce_key = std::make_unique<Botan::Classic_McEliece_PublicKey>(std::span{pubkey, key_len}, mode);
+      return ffi_new_object(key, std::move(cmce_key));
+   });
+#else
+   BOTAN_UNUSED(key, pubkey, key_len, cmce_mode);
+   return BOTAN_FFI_ERROR_NOT_IMPLEMENTED;
+#endif
+}
+
 int botan_pubkey_view_ec_public_point(const botan_pubkey_t key, botan_view_ctx ctx, botan_view_bin_fn view) {
 #if defined(BOTAN_HAS_ECC_PUBLIC_KEY_CRYPTO)
    return BOTAN_FFI_VISIT(key, [=](const auto& k) -> int {
       if(auto ecc = dynamic_cast<const Botan::EC_PublicKey*>(&k)) {
-         auto pt = ecc->public_point().encode(Botan::EC_Point_Format::Uncompressed);
+         auto pt = ecc->_public_ec_point().serialize_uncompressed();
          return invoke_view_callback(view, ctx, pt);
       } else {
          return BOTAN_FFI_ERROR_BAD_PARAMETER;
@@ -1259,6 +1366,8 @@ int botan_pubkey_view_ec_public_point(const botan_pubkey_t key, botan_view_ctx c
 #endif
 }
 
+// NOLINTEND(misc-misplaced-const)
+
 int botan_privkey_create_mceliece(botan_privkey_t* key_obj, botan_rng_t rng_obj, size_t n, size_t t) {
    const std::string mce_params = std::to_string(n) + "," + std::to_string(t);
    return botan_privkey_create(key_obj, "McEliece", mce_params.c_str(), rng_obj);
