@@ -143,7 +143,7 @@ void Streebog::compress_64(const uint64_t M[], bool last_block) {
       hN[i] ^= M[i];
    }
 
-   for(size_t i = 0; i < 12; ++i) {
+   for(size_t i = 0; i < 12; ++i) {  // NOLINT(modernize-loop-convert)
       for(size_t j = 0; j != 8; ++j) {
          A[j] ^= force_le(STREEBOG_C[i][j]);
       }
@@ -168,7 +168,7 @@ void Streebog::compress_64(const uint64_t M[], bool last_block) {
 
          m_S[i] = force_le(t);
          if(t != m) {
-            carry = (t < m);
+            carry = (t < m) ? 1 : 0;
          }
       }
    }
