@@ -20,6 +20,8 @@
 
 namespace Botan {
 
+// NOLINTBEGIN(*-macro-usage)
+
 #if defined(BOTAN_USE_GCC_INLINE_ASM) && defined(BOTAN_TARGET_ARCH_IS_X86_64)
    #define BOTAN_MP_USE_X86_64_ASM
 #endif
@@ -322,32 +324,6 @@ inline constexpr auto word8_sub2(W x[8], const W y[8], W carry) -> W {
    return carry;
 }
 
-/*
-* Eight Word Block Subtraction, Two Argument
-*/
-template <WordType W>
-inline constexpr auto word8_sub2_rev(W x[8], const W y[8], W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_64_ASM)
-   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(ADD_OR_SUBTRACT(DO_8_TIMES(ADDSUB3_OP, "sbbq"))
-          : [carry] "=r"(carry)
-          : [x] "r"(y), [y] "r"(x), [z] "r"(x), "0"(carry)
-          : "cc", "memory");
-      return carry;
-   }
-#endif
-
-   x[0] = word_sub(y[0], x[0], &carry);
-   x[1] = word_sub(y[1], x[1], &carry);
-   x[2] = word_sub(y[2], x[2], &carry);
-   x[3] = word_sub(y[3], x[3], &carry);
-   x[4] = word_sub(y[4], x[4], &carry);
-   x[5] = word_sub(y[5], x[5], &carry);
-   x[6] = word_sub(y[6], x[6], &carry);
-   x[7] = word_sub(y[7], x[7], &carry);
-   return carry;
-}
-
 /*
 * Eight Word Block Subtraction, Three Argument
 */
@@ -374,32 +350,6 @@ inline constexpr auto word8_sub3(W z[8], const W x[8], const W y[8], W carry) ->
    return carry;
 }
 
-/*
-* Eight Word Block Linear Multiplication
-*/
-template <WordType W>
-inline constexpr auto word8_linmul2(W x[8], W y, W carry) -> W {
-#if defined(BOTAN_MP_USE_X86_64_ASM)
-   if(std::same_as<W, uint64_t> && !std::is_constant_evaluated()) {
-      asm(DO_8_TIMES(LINMUL_OP, "x")
-          : [carry] "=r"(carry)
-          : [x] "r"(x), [y] "rm"(y), "0"(carry)
-          : "cc", "%rax", "%rdx");
-      return carry;
-   }
-#endif
-
-   x[0] = word_madd2(x[0], y, &carry);
-   x[1] = word_madd2(x[1], y, &carry);
-   x[2] = word_madd2(x[2], y, &carry);
-   x[3] = word_madd2(x[3], y, &carry);
-   x[4] = word_madd2(x[4], y, &carry);
-   x[5] = word_madd2(x[5], y, &carry);
-   x[6] = word_madd2(x[6], y, &carry);
-   x[7] = word_madd2(x[7], y, &carry);
-   return carry;
-}
-
 /*
 * Eight Word Block Linear Multiplication
 */
@@ -465,7 +415,7 @@ class word3 final {
 #if defined(__BITINT_MAXWIDTH__) && (__BITINT_MAXWIDTH__ >= 3 * 64)
 
    public:
-      constexpr word3() { m_w = 0; }
+      constexpr word3() : m_w(0) {}
 
       inline constexpr void mul(W x, W y) { m_w += static_cast<W3>(x) * y; }
 
@@ -500,11 +450,7 @@ class word3 final {
 #else
 
    public:
-      constexpr word3() {
-         m_w2 = 0;
-         m_w1 = 0;
-         m_w0 = 0;
-      }
+      constexpr word3() : m_w0(0), m_w1(0), m_w2(0) {}
 
       inline constexpr void mul(W x, W y) {
    #if defined(BOTAN_MP_USE_X86_64_ASM)
@@ -607,7 +553,9 @@ class word3 final {
       }
 
    private:
-      W m_w0, m_w1, m_w2;
+      W m_w0;
+      W m_w1;
+      W m_w2;
 #endif
 };
 
@@ -621,6 +569,8 @@ class word3 final {
    #undef MULADD_OP
 #endif
 
+// NOLINTEND(*-macro-usage)
+
 }  // namespace Botan
 
 #endif
