@@ -7,20 +7,20 @@
 
 #include <botan/srp6.h>
 
+#include <botan/assert.h>
 #include <botan/dl_group.h>
 #include <botan/hash.h>
-#include <botan/numthry.h>
 #include <botan/internal/fmt.h>
 
 namespace Botan {
 
 namespace {
 
-BigInt hash_seq(HashFunction& hash_fn, size_t pad_to, const BigInt& in1, const BigInt& in2) {
-   hash_fn.update(BigInt::encode_1363(in1, pad_to));
-   hash_fn.update(BigInt::encode_1363(in2, pad_to));
+BigInt hash_seq(HashFunction& hash_fn, size_t p_bytes, const BigInt& in1, const BigInt& in2) {
+   hash_fn.update(in1.serialize(p_bytes));
+   hash_fn.update(in2.serialize(p_bytes));
 
-   return BigInt::decode(hash_fn.final());
+   return BigInt::from_bytes(hash_fn.final());
 }
 
 BigInt compute_x(HashFunction& hash_fn,
@@ -38,7 +38,7 @@ BigInt compute_x(HashFunction& hash_fn,
 
    secure_vector<uint8_t> outer_h = hash_fn.final();
 
-   return BigInt::decode(outer_h);
+   return BigInt::from_bytes(outer_h);
 }
 
 }  // namespace
@@ -49,9 +49,9 @@ std::string srp6_group_identifier(const BigInt& N, const BigInt& g) {
    been defined for a particular bitsize. As of this writing that is the case.
    */
    try {
-      std::string group_name = "modp/srp/" + std::to_string(N.bits());
+      const std::string group_name = "modp/srp/" + std::to_string(N.bits());
 
-      DL_Group group(group_name);
+      auto group = DL_Group::from_name(group_name);
 
       if(group.get_p() == N && group.get_g() == g) {
          return group_name;
@@ -69,7 +69,7 @@ std::pair<BigInt, SymmetricKey> srp6_client_agree(std::string_view identifier,
                                                   const std::vector<uint8_t>& salt,
                                                   const BigInt& B,
                                                   RandomNumberGenerator& rng) {
-   DL_Group group(group_id);
+   auto group = DL_Group::from_name(group_id);
    const size_t a_bits = group.exponent_bits();
 
    return srp6_client_agree(identifier, password, group, hash_id, salt, B, a_bits, rng);
@@ -111,7 +111,7 @@ std::pair<BigInt, SymmetricKey> srp6_client_agree(std::string_view identifier,
 
    const BigInt g_x_p = group.power_g_p(x, hash_fn->output_length() * 8);
 
-   const BigInt B_k_g_x_p = group.mod_p(B - group.multiply_mod_p(k, g_x_p));
+   const BigInt B_k_g_x_p = group.mod_p(B + group.mod_p(p - group.multiply_mod_p(k, g_x_p)));
 
    const BigInt a_ux = a + u * x;
 
@@ -120,7 +120,7 @@ std::pair<BigInt, SymmetricKey> srp6_client_agree(std::string_view identifier,
 
    const BigInt S = group.power_b_p(B_k_g_x_p, a_ux, max_aux_bits);
 
-   const SymmetricKey Sk(BigInt::encode_1363(S, p_bytes));
+   const SymmetricKey Sk(S.serialize<secure_vector<uint8_t>>(p_bytes));
 
    return std::make_pair(A, Sk);
 }
@@ -130,7 +130,7 @@ BigInt srp6_generate_verifier(std::string_view identifier,
                               const std::vector<uint8_t>& salt,
                               std::string_view group_id,
                               std::string_view hash_id) {
-   DL_Group group(group_id);
+   auto group = DL_Group::from_name(group_id);
    return srp6_generate_verifier(identifier, password, salt, group, hash_id);
 }
 
@@ -152,7 +152,7 @@ BigInt SRP6_Server_Session::step1(const BigInt& v,
                                   std::string_view group_id,
                                   std::string_view hash_id,
                                   RandomNumberGenerator& rng) {
-   DL_Group group(group_id);
+   auto group = DL_Group::from_name(group_id);
    const size_t b_bits = group.exponent_bits();
    return this->step1(v, group, hash_id, b_bits, rng);
 }
@@ -161,42 +161,45 @@ BigInt SRP6_Server_Session::step1(
    const BigInt& v, const DL_Group& group, std::string_view hash_id, size_t b_bits, RandomNumberGenerator& rng) {
    BOTAN_ARG_CHECK(b_bits <= group.p_bits(), "Invalid b_bits");
 
-   m_group = group;
+   BOTAN_STATE_CHECK(!m_group);
+   m_group = std::make_unique<DL_Group>(group);
 
-   const BigInt& g = group.get_g();
-   const BigInt& p = group.get_p();
+   const BigInt& g = m_group->get_g();
+   const BigInt& p = m_group->get_p();
 
    m_v = v;
    m_b = BigInt(rng, b_bits);
    m_hash_id = hash_id;
 
    auto hash_fn = HashFunction::create_or_throw(hash_id);
-   if(8 * hash_fn->output_length() >= group.p_bits()) {
+   if(8 * hash_fn->output_length() >= m_group->p_bits()) {
       throw Invalid_Argument(fmt("Hash function {} too large for SRP6 with this group", hash_fn->name()));
    }
 
-   const BigInt k = hash_seq(*hash_fn, m_group.p_bytes(), p, g);
-   m_B = group.mod_p(v * k + group.power_g_p(m_b, b_bits));
+   const BigInt k = hash_seq(*hash_fn, m_group->p_bytes(), p, g);
+   m_B = m_group->mod_p(v * k + m_group->power_g_p(m_b, b_bits));
 
    return m_B;
 }
 
 SymmetricKey SRP6_Server_Session::step2(const BigInt& A) {
-   if(A <= 0 || A >= m_group.get_p()) {
+   BOTAN_STATE_CHECK(m_group);
+
+   if(A <= 0 || A >= m_group->get_p()) {
       throw Decoding_Error("Invalid SRP parameter from client");
    }
 
    auto hash_fn = HashFunction::create_or_throw(m_hash_id);
-   if(8 * hash_fn->output_length() >= m_group.p_bits()) {
+   if(8 * hash_fn->output_length() >= m_group->p_bits()) {
       throw Invalid_Argument(fmt("Hash function {} too large for SRP6 with this group", hash_fn->name()));
    }
 
-   const BigInt u = hash_seq(*hash_fn, m_group.p_bytes(), A, m_B);
+   const BigInt u = hash_seq(*hash_fn, m_group->p_bytes(), A, m_B);
 
-   const BigInt vup = m_group.power_b_p(m_v, u, m_group.p_bits());
-   const BigInt S = m_group.power_b_p(m_group.multiply_mod_p(A, vup), m_b, m_group.p_bits());
+   const BigInt vup = m_group->power_b_p(m_v, u, m_group->p_bits());
+   const BigInt S = m_group->power_b_p(m_group->multiply_mod_p(A, vup), m_b, m_group->p_bits());
 
-   return SymmetricKey(BigInt::encode_1363(S, m_group.p_bytes()));
+   return SymmetricKey(S.serialize<secure_vector<uint8_t>>(m_group->p_bytes()));
 }
 
 }  // namespace Botan
