@@ -31,7 +31,6 @@
    #include <boost/asio.hpp>
    #include <boost/beast/core.hpp>
 
-   #include <algorithm>
    #include <memory>
    #include <type_traits>
 
@@ -57,14 +56,14 @@ class Stream;
  */
 class StreamCallbacks : public Callbacks {
    public:
-      StreamCallbacks() {}
+      StreamCallbacks() = default;
 
       void tls_emit_data(std::span<const uint8_t> data) final {
          m_send_buffer.commit(boost::asio::buffer_copy(m_send_buffer.prepare(data.size()),
                                                        boost::asio::buffer(data.data(), data.size())));
       }
 
-      void tls_record_received(uint64_t, std::span<const uint8_t> data) final {
+      void tls_record_received(uint64_t /*record_number*/, std::span<const uint8_t> data) final {
          m_receive_buffer.commit(boost::asio::buffer_copy(m_receive_buffer.prepare(data.size()),
                                                           boost::asio::const_buffer(data.data(), data.size())));
       }
@@ -242,7 +241,7 @@ class Stream {
       //! \name boost::asio accessor methods
       //! @{
 
-      using next_layer_type = typename std::remove_reference<StreamLayer>::type;
+      using next_layer_type = std::remove_reference_t<StreamLayer>;
 
       const next_layer_type& next_layer() const { return m_nextLayer; }
 
@@ -258,10 +257,12 @@ class Stream {
 
       executor_type get_executor() noexcept { return m_nextLayer.get_executor(); }
 
-      using native_handle_type = typename std::add_pointer<ChannelT>::type;
+      using native_handle_type = std::add_pointer_t<ChannelT>;
 
       native_handle_type native_handle() {
-         BOTAN_STATE_CHECK(m_native_handle != nullptr);
+         if(m_native_handle == nullptr) {
+            throw Botan::Invalid_State("ASIO native handle unexpectedly null");
+         }
          return m_native_handle.get();
       }
 
@@ -288,8 +289,7 @@ class Stream {
        * @param callback the callback implementation
        * @param ec This parameter is unused.
        */
-      void set_verify_callback(Context::Verify_Callback callback, boost::system::error_code& ec) {
-         BOTAN_UNUSED(ec);
+      void set_verify_callback(Context::Verify_Callback callback, [[maybe_unused]] boost::system::error_code& ec) {
          m_context->set_verify_callback(std::move(callback));
       }
 
@@ -298,8 +298,7 @@ class Stream {
        * @param depth the desired verification depth
        * @throws Not_Implemented todo
        */
-      void set_verify_depth(int depth) {
-         BOTAN_UNUSED(depth);
+      void set_verify_depth([[maybe_unused]] int depth) {
          throw Not_Implemented("set_verify_depth is not implemented");
       }
 
@@ -308,8 +307,7 @@ class Stream {
        * @param depth the desired verification depth
        * @param ec Will be set to `Botan::ErrorType::NotImplemented`
        */
-      void set_verify_depth(int depth, boost::system::error_code& ec) {
-         BOTAN_UNUSED(depth);
+      void set_verify_depth([[maybe_unused]] int depth, boost::system::error_code& ec) {
          ec = ErrorType::NotImplemented;
       }
 
@@ -319,8 +317,7 @@ class Stream {
        * @throws Not_Implemented todo
        */
       template <typename verify_mode>
-      void set_verify_mode(verify_mode v) {
-         BOTAN_UNUSED(v);
+      void set_verify_mode([[maybe_unused]] verify_mode v) {
          throw Not_Implemented("set_verify_mode is not implemented");
       }
 
@@ -330,8 +327,7 @@ class Stream {
        * @param ec Will be set to `Botan::ErrorType::NotImplemented`
        */
       template <typename verify_mode>
-      void set_verify_mode(verify_mode v, boost::system::error_code& ec) {
-         BOTAN_UNUSED(v);
+      void set_verify_mode([[maybe_unused]] verify_mode v, boost::system::error_code& ec) {
          ec = ErrorType::NotImplemented;
       }
 
@@ -393,8 +389,6 @@ class Stream {
             // handled by `handle_tls_protocol_errors()` in the next iteration.
             read_and_process_encrypted_data_from_peer(ec);
          }
-
-         BOTAN_ASSERT_NOMSG(ec.failed());
       }
 
       /**
@@ -408,6 +402,7 @@ class Stream {
        */
       template <detail::basic_completion_token CompletionToken = default_completion_token>
       auto async_handshake(Botan::TLS::Connection_Side side,
+                           // NOLINTNEXTLINE(*-missing-std-forward)
                            CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code)>(
             [this](auto&& completion_handler, TLS::Connection_Side connection_side) {
@@ -428,10 +423,9 @@ class Stream {
        * @throws Not_Implemented todo
        */
       template <typename ConstBufferSequence, detail::basic_completion_token BufferedHandshakeHandler>
-      auto async_handshake(Connection_Side side,
-                           const ConstBufferSequence& buffers,
-                           BufferedHandshakeHandler&& handler) {
-         BOTAN_UNUSED(side, buffers, handler);
+      auto async_handshake([[maybe_unused]] Connection_Side side,
+                           [[maybe_unused]] const ConstBufferSequence& buffers,
+                           [[maybe_unused]] BufferedHandshakeHandler&& handler /* NOLINT(*missing-std-forward) */) {
          throw Not_Implemented("buffered async handshake is not implemented");
       }
 
@@ -481,7 +475,7 @@ class Stream {
        */
       template <typename Handler, typename Executor>
       struct Wrapper {
-            void operator()(boost::system::error_code ec, std::size_t) { handler(ec); }
+            void operator()(boost::system::error_code ec, std::size_t /*unused*/) { handler(ec); }
 
             using executor_type = boost::asio::associated_executor_t<Handler, Executor>;
 
@@ -493,8 +487,8 @@ class Stream {
 
             allocator_type get_allocator() const noexcept { return boost::asio::get_associated_allocator(handler); }
 
-            Handler handler;
-            Executor io_executor;
+            Handler handler;       // NOLINT(*-non-private-member-variable*)
+            Executor io_executor;  // NOLINT(*-non-private-member-variable*)
       };
 
    public:
@@ -509,6 +503,7 @@ class Stream {
        *                         The completion signature of the handler must be: void(boost::system::error_code).
        */
       template <detail::basic_completion_token CompletionToken = default_completion_token>
+      // NOLINTNEXTLINE(*-missing-std-forward)
       auto async_shutdown(CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code)>(
             [this](auto&& completion_handler) {
@@ -570,7 +565,6 @@ class Stream {
             read_and_process_encrypted_data_from_peer(ec);
          }
 
-         BOTAN_ASSERT_NOMSG(ec.failed());
          return 0;
       }
 
@@ -638,6 +632,7 @@ class Stream {
       template <typename ConstBufferSequence,
                 detail::byte_size_completion_token CompletionToken = default_completion_token>
       auto async_write_some(const ConstBufferSequence& buffers,
+                            // NOLINTNEXTLINE(*-missing-std-forward)
                             CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code, std::size_t)>(
             [this](auto&& completion_handler, const auto& bufs) {
@@ -674,6 +669,7 @@ class Stream {
       template <typename MutableBufferSequence,
                 detail::byte_size_completion_token CompletionToken = default_completion_token>
       auto async_read_some(const MutableBufferSequence& buffers,
+                           // NOLINTNEXTLINE(*-missing-std-forward)
                            CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code, std::size_t)>(
             [this](auto&& completion_handler, const auto& bufs) {
@@ -739,8 +735,10 @@ class Stream {
       void setup_native_handle(Connection_Side side, boost::system::error_code& ec) {
          // Do not attempt to instantiate the native_handle when a custom (mocked) channel type template parameter has
          // been specified. This allows mocking the native_handle in test code.
-         if constexpr(std::is_same<ChannelT, Channel>::value) {
-            BOTAN_STATE_CHECK(m_native_handle == nullptr);
+         if constexpr(std::is_same_v<ChannelT, Channel>) {
+            if(m_native_handle != nullptr) {
+               throw Botan::Invalid_State("ASIO native handle unexpectedly set");
+            }
 
             try_with_error_code(
                [&] {
@@ -829,8 +827,13 @@ class Stream {
          // If we have received application data in a previous invocation, this
          // data needs to be passed to the application first. Otherwise, it
          // might get overwritten.
-         BOTAN_ASSERT(!has_received_data(), "receive buffer is empty");
-         BOTAN_ASSERT(!error_from_us() && !alert_from_peer(), "TLS session is healthy");
+         if(has_received_data()) {
+            throw Botan::Invalid_State("ASIO receive buffer not empty");
+         }
+
+         if(error_from_us() || alert_from_peer()) {
+            throw Botan::Invalid_State("ASIO TLS session no longer healthy");
+         }
 
          // If there's no existing error condition, read and process data from
          // the peer and report any sort of network error. TLS related errors do
@@ -913,8 +916,9 @@ class Stream {
        * @param read_buffer Input buffer containing the encrypted data.
        */
       void process_encrypted_data(const boost::asio::const_buffer& read_buffer) {
-         BOTAN_ASSERT(!alert_from_peer() && !error_from_us(),
-                      "no one sent an alert before (no data allowed after that)");
+         if(alert_from_peer() || error_from_us()) {
+            throw Botan::Invalid_State("ASIO TLS session no longer healthy");
+         }
 
          // If the local TLS implementation generates an alert, we are notified
          // with an exception that is caught in try_with_error_code(). The error
@@ -958,16 +962,16 @@ class Stream {
       boost::system::error_code error_from_us() const { return m_ec_from_last_read; }
 
    protected:
-      std::shared_ptr<Context> m_context;
-      StreamLayer m_nextLayer;
+      std::shared_ptr<Context> m_context;  // NOLINT(*-non-private-member-variable*)
+      StreamLayer m_nextLayer;             // NOLINT(*-non-private-member-variable*)
 
-      std::shared_ptr<StreamCallbacks> m_core;
-      std::unique_ptr<ChannelT> m_native_handle;
-      boost::system::error_code m_ec_from_last_read;
+      std::shared_ptr<StreamCallbacks> m_core;        // NOLINT(*-non-private-member-variable*)
+      std::unique_ptr<ChannelT> m_native_handle;      // NOLINT(*-non-private-member-variable*)
+      boost::system::error_code m_ec_from_last_read;  // NOLINT(*-non-private-member-variable*)
 
       // Buffer space used to read input intended for the core
-      std::vector<uint8_t> m_input_buffer_space;
-      const boost::asio::mutable_buffer m_input_buffer;
+      std::vector<uint8_t> m_input_buffer_space;         // NOLINT(*-non-private-member-variable*)
+      const boost::asio::mutable_buffer m_input_buffer;  // NOLINT(*-non-private-member-variable*)
 };
 
 // deduction guides for convenient construction from an existing
