@@ -20,6 +20,10 @@
 
 namespace Botan::TLS {
 
+bool Policy::allow_ssl_key_log_file() const {
+   return false;
+}
+
 std::vector<Signature_Scheme> Policy::allowed_signature_schemes() const {
    std::vector<Signature_Scheme> schemes;
 
@@ -47,9 +51,9 @@ std::optional<std::vector<Signature_Scheme>> Policy::acceptable_certificate_sign
 std::vector<std::string> Policy::allowed_ciphers() const {
    return {
       //"AES-256/OCB(12)",
-      "ChaCha20Poly1305",
       "AES-256/GCM",
       "AES-128/GCM",
+      "ChaCha20Poly1305",
       //"AES-256/CCM",
       //"AES-128/CCM",
       //"AES-256/CCM(8)",
@@ -121,7 +125,21 @@ Group_Params Policy::choose_key_exchange_group(const std::vector<Group_Params>&
       return Group_Params::NONE;
    }
 
-   const std::vector<Group_Params> our_groups = key_exchange_groups();
+   const auto our_groups = key_exchange_groups();
+
+   // First check if the peer sent a PQ share of a group we also support
+   for(auto share : offered_by_peer) {
+      if(share.is_post_quantum() && value_exists(our_groups, share)) {
+         return share;
+      }
+   }
+
+   // Then check if the peer offered a PQ algo we also support
+   for(auto share : supported_by_peer) {
+      if(share.is_post_quantum() && value_exists(our_groups, share)) {
+         return share;
+      }
+   }
 
    // Prefer groups that were offered by the peer for the sake of saving
    // an additional round trip. For TLS 1.2, this won't be used.
@@ -156,27 +174,68 @@ Group_Params Policy::default_dh_group() const {
 }
 
 std::vector<Group_Params> Policy::key_exchange_groups() const {
-   // Default list is ordered by performance
    return {
-#if defined(BOTAN_HAS_CURVE_25519)
+      // clang-format off
+#if defined(BOTAN_HAS_X25519)
       Group_Params::X25519,
 #endif
 
-         Group_Params::SECP256R1, Group_Params::BRAINPOOL256R1, Group_Params::SECP384R1, Group_Params::BRAINPOOL384R1,
-         Group_Params::SECP521R1, Group_Params::BRAINPOOL512R1,
+      Group_Params::SECP256R1,
 
-         Group_Params::FFDHE_2048, Group_Params::FFDHE_3072, Group_Params::FFDHE_4096, Group_Params::FFDHE_6144,
-         Group_Params::FFDHE_8192,
+#if defined(BOTAN_HAS_ML_KEM) && defined(BOTAN_HAS_TLS_13_PQC)
+
+#if defined(BOTAN_HAS_X25519)
+      Group_Params_Code::HYBRID_X25519_ML_KEM_768,
+#endif
+
+      Group_Params_Code::HYBRID_SECP256R1_ML_KEM_768,
+      Group_Params_Code::HYBRID_SECP384R1_ML_KEM_1024,
+#endif
+
+#if defined(BOTAN_HAS_X448)
+      Group_Params::X448,
+#endif
+
+      Group_Params::SECP384R1,
+      Group_Params::SECP521R1,
+
+      Group_Params::BRAINPOOL256R1,
+      Group_Params::BRAINPOOL384R1,
+      Group_Params::BRAINPOOL512R1,
+
+      Group_Params::FFDHE_2048,
+      Group_Params::FFDHE_3072,
+
+      // clang-format on
    };
 }
 
 std::vector<Group_Params> Policy::key_exchange_groups_to_offer() const {
-   // by default, we offer a key share for the most-preferred group, only
    std::vector<Group_Params> groups_to_offer;
+
    const auto supported_groups = key_exchange_groups();
-   if(!supported_groups.empty()) {
+   BOTAN_ASSERT(!supported_groups.empty(), "Policy allows at least one key exchange group");
+
+   /*
+   * Initially prefer sending a key share only of the first pure-ECC
+   * group, since these shares are small and PQ support is still not
+   * that widespread.
+   */
+   for(auto group : key_exchange_groups()) {
+      if(group.is_pure_ecc_group()) {
+         groups_to_offer.push_back(group);
+         break;
+      }
+   }
+
+   /*
+   * If for some reason no pure ECC groups are enabled then simply
+   * send a share of whatever the policys top preference is.
+   */
+   if(groups_to_offer.empty()) {
       groups_to_offer.push_back(supported_groups.front());
    }
+
    return groups_to_offer;
 }
 
@@ -223,7 +282,7 @@ void Policy::check_peer_key_acceptable(const Public_Key& public_key) const {
       expected_keylength = minimum_rsa_bits();
    } else if(algo_name == "DH") {
       expected_keylength = minimum_dh_group_size();
-   } else if(algo_name == "ECDH" || algo_name == "Curve25519") {
+   } else if(algo_name == "ECDH" || algo_name == "X25519" || algo_name == "X448") {
       expected_keylength = minimum_ecdh_group_size();
    } else if(algo_name == "ECDSA") {
       expected_keylength = minimum_ecdsa_group_size();
@@ -387,6 +446,14 @@ bool Policy::abort_connection_on_undesired_renegotiation() const {
    return false;
 }
 
+std::vector<Certificate_Type> Policy::accepted_client_certificate_types() const {
+   return {Certificate_Type::X509};
+}
+
+std::vector<Certificate_Type> Policy::accepted_server_certificate_types() const {
+   return {Certificate_Type::X509};
+}
+
 bool Policy::allow_dtls_epoch0_restart() const {
    return false;
 }
@@ -585,6 +652,17 @@ void print_vec(std::ostream& o, const char* key, const std::vector<Group_Params>
    o << "\n";
 }
 
+void print_vec(std::ostream& o, const char* key, const std::vector<Certificate_Type>& types) {
+   o << key << " = ";
+   for(size_t i = 0; i != types.size(); ++i) {
+      o << certificate_type_to_string(types[i]);
+      if(i != types.size() - 1) {
+         o << ' ';
+      }
+   }
+   o << '\n';
+}
+
 void print_bool(std::ostream& o, const char* key, bool b) {
    o << key << " = " << (b ? "true" : "false") << '\n';
 }
@@ -595,6 +673,7 @@ void Policy::print(std::ostream& o) const {
    print_bool(o, "allow_tls12", allow_tls12());
    print_bool(o, "allow_tls13", allow_tls13());
    print_bool(o, "allow_dtls12", allow_dtls12());
+   print_bool(o, "allow_ssl_key_log_file", allow_ssl_key_log_file());
    print_vec(o, "ciphers", allowed_ciphers());
    print_vec(o, "macs", allowed_macs());
    print_vec(o, "signature_hashes", allowed_signature_hashes());
@@ -615,13 +694,15 @@ void Policy::print(std::ostream& o) const {
    print_bool(o, "negotiate_encrypt_then_mac", negotiate_encrypt_then_mac());
    print_bool(o, "support_cert_status_message", support_cert_status_message());
    print_bool(o, "tls_13_middlebox_compatibility_mode", tls_13_middlebox_compatibility_mode());
+   print_vec(o, "accepted_client_certificate_types", accepted_client_certificate_types());
+   print_vec(o, "accepted_server_certificate_types", accepted_server_certificate_types());
    print_bool(o, "hash_hello_random", hash_hello_random());
    if(record_size_limit().has_value()) {
       o << "record_size_limit = " << record_size_limit().value() << '\n';
    }
    o << "maximum_session_tickets_per_client_hello = " << maximum_session_tickets_per_client_hello() << '\n';
    o << "session_ticket_lifetime = " << session_ticket_lifetime().count() << '\n';
-   o << "reuse_session_tickets = " << reuse_session_tickets() << '\n';
+   print_bool(o, "reuse_session_tickets", reuse_session_tickets());
    o << "new_session_tickets_upon_handshake_success = " << new_session_tickets_upon_handshake_success() << '\n';
    o << "minimum_dh_group_size = " << minimum_dh_group_size() << '\n';
    o << "minimum_ecdh_group_size = " << minimum_ecdh_group_size() << '\n';
@@ -636,7 +717,7 @@ std::string Policy::to_string() const {
 }
 
 std::vector<std::string> Strict_Policy::allowed_ciphers() const {
-   return {"ChaCha20Poly1305", "AES-256/GCM", "AES-128/GCM"};
+   return {"AES-256/GCM", "AES-128/GCM", "ChaCha20Poly1305"};
 }
 
 std::vector<std::string> Strict_Policy::allowed_signature_hashes() const {
