@@ -14,11 +14,11 @@
 #include <botan/mac.h>
 #include <botan/pem.h>
 #include <botan/rng.h>
-#include <botan/internal/loadstor.h>
-
 #include <botan/tls_callbacks.h>
 #include <botan/tls_messages.h>
-
+#include <botan/x509_key.h>
+#include <botan/internal/ct_utils.h>
+#include <botan/internal/loadstor.h>
 #include <botan/internal/stl_util.h>
 
 #include <utility>
@@ -103,9 +103,47 @@ Session_Summary::Session_Summary(const Session_Base& base,
 
 #if defined(BOTAN_HAS_TLS_13)
 
+namespace {
+
+std::string tls13_kex_to_string(bool psk, std::optional<Named_Group> group) {
+   if(psk && group) {
+      if(group->is_dh_named_group()) {
+         return kex_method_to_string(Kex_Algo::DHE_PSK);
+      } else if(group->is_ecdh_named_curve() || group->is_x25519() || group->is_x448()) {
+         return kex_method_to_string(Kex_Algo::ECDHE_PSK);
+      } else if(group->is_pure_ml_kem() || group->is_pure_frodokem()) {
+         return kex_method_to_string(Kex_Algo::KEM_PSK);
+      } else if(group->is_pqc_hybrid()) {
+         return kex_method_to_string(Kex_Algo::HYBRID_PSK);
+      } else if(auto s = group->to_string()) {
+         return *s;
+      }
+   } else if(psk) {
+      return kex_method_to_string(Kex_Algo::PSK);
+   } else {
+      BOTAN_ASSERT_NOMSG(group.has_value());
+      if(group->is_dh_named_group()) {
+         return kex_method_to_string(Kex_Algo::DH);
+      } else if(group->is_ecdh_named_curve() || group->is_x25519() || group->is_x448()) {
+         return kex_method_to_string(Kex_Algo::ECDH);
+      } else if(group->is_pure_ml_kem() || group->is_pure_frodokem()) {
+         return kex_method_to_string(Kex_Algo::KEM);
+      } else if(group->is_pqc_hybrid()) {
+         return kex_method_to_string(Kex_Algo::HYBRID);
+      } else if(auto s = group->to_string()) {
+         return *s;
+      }
+   }
+
+   return kex_method_to_string(Kex_Algo::UNDEFINED);
+}
+
+}  // namespace
+
 Session_Summary::Session_Summary(const Server_Hello_13& server_hello,
                                  Connection_Side side,
                                  std::vector<X509_Certificate> peer_certs,
+                                 std::shared_ptr<const Public_Key> peer_raw_public_key,
                                  std::optional<std::string> psk_identity,
                                  bool session_was_resumed,
                                  Server_Information server_info,
@@ -128,6 +166,7 @@ Session_Summary::Session_Summary(const Server_Hello_13& server_hello,
                    // TLS 1.3 uses AEADs, so technically encrypt-then-MAC is not applicable.
                    false,
                    std::move(peer_certs),
+                   std::move(peer_raw_public_key),
                    std::move(server_info)),
       m_external_psk_identity(std::move(psk_identity)),
       m_was_resumption(session_was_resumed) {
@@ -136,39 +175,26 @@ Session_Summary::Session_Summary(const Server_Hello_13& server_hello,
 
    // In TLS 1.3 the key exchange algorithm is not negotiated in the ciphersuite
    // anymore. This provides a compatible identifier for applications to use.
-   m_kex_algo = kex_method_to_string([&] {
+
+   std::optional<Named_Group> group = [&]() -> std::optional<Named_Group> {
       if(psk_used() || was_resumption()) {
-         if(const auto keyshare = server_hello.extensions().get<Key_Share>()) {
-            const auto group = keyshare->selected_group();
-            if(group.is_dh_named_group()) {
-               return Kex_Algo::DHE_PSK;
-            } else if(group.is_ecdh_named_curve() || group.is_x25519()) {
-               return Kex_Algo::ECDHE_PSK;
-            } else if(group.is_pure_kyber()) {
-               return Kex_Algo::KEM_PSK;
-            } else if(group.is_pqc_hybrid()) {
-               return Kex_Algo::HYBRID_PSK;
-            }
+         if(auto* const keyshare = server_hello.extensions().get<Key_Share>()) {
+            return keyshare->selected_group();
          } else {
-            return Kex_Algo::PSK;
+            return {};
          }
       } else {
-         const auto keyshare = server_hello.extensions().get<Key_Share>();
+         auto* const keyshare = server_hello.extensions().get<Key_Share>();
          BOTAN_ASSERT_NONNULL(keyshare);
-         const auto group = keyshare->selected_group();
-         if(group.is_dh_named_group()) {
-            return Kex_Algo::DH;
-         } else if(group.is_ecdh_named_curve() || group.is_x25519()) {
-            return Kex_Algo::ECDH;
-         } else if(group.is_pure_kyber()) {
-            return Kex_Algo::KEM;
-         } else if(group.is_pqc_hybrid()) {
-            return Kex_Algo::HYBRID;
-         }
+         return keyshare->selected_group();
       }
+   }();
 
-      return Kex_Algo::UNDEFINED;
-   }());
+   if(group.has_value()) {
+      m_kex_parameters = group->to_string();
+   }
+
+   m_kex_algo = tls13_kex_to_string(psk_used() || was_resumption(), group);
 }
 
 #endif
@@ -192,6 +218,7 @@ Session::Session(const secure_vector<uint8_t>& master_secret,
                    extended_master_secret,
                    encrypt_then_mac,
                    certs,
+                   nullptr,  // RFC 7250 (raw public keys) is NYI for TLS 1.2
                    server_info),
       m_master_secret(master_secret),
       m_early_data_allowed(false),
@@ -211,6 +238,7 @@ Session::Session(const secure_vector<uint8_t>& session_psk,
                  uint16_t ciphersuite,
                  Connection_Side side,
                  const std::vector<X509_Certificate>& peer_certs,
+                 std::shared_ptr<const Public_Key> peer_raw_public_key,
                  const Server_Information& server_info,
                  std::chrono::system_clock::time_point current_timestamp) :
       Session_Base(current_timestamp,
@@ -231,6 +259,7 @@ Session::Session(const secure_vector<uint8_t>& session_psk,
                    // TLS 1.3 uses AEADs, so technically encrypt-then-MAC is not applicable.
                    false,
                    peer_certs,
+                   std::move(peer_raw_public_key),
                    server_info),
       m_master_secret(session_psk),
       m_early_data_allowed(max_early_data_bytes.has_value()),
@@ -244,6 +273,7 @@ Session::Session(secure_vector<uint8_t>&& session_psk,
                  const std::optional<uint32_t>& max_early_data_bytes,
                  std::chrono::seconds lifetime_hint,
                  const std::vector<X509_Certificate>& peer_certs,
+                 std::shared_ptr<const Public_Key> peer_raw_public_key,
                  const Client_Hello_13& client_hello,
                  const Server_Hello_13& server_hello,
                  Callbacks& callbacks,
@@ -256,6 +286,7 @@ Session::Session(secure_vector<uint8_t>&& session_psk,
                    true,
                    false,  // see constructor above for rationales
                    peer_certs,
+                   std::move(peer_raw_public_key),
                    Server_Information(client_hello.sni_hostname())),
       m_master_secret(std::move(session_psk)),
       m_early_data_allowed(max_early_data_bytes.has_value()),
@@ -270,14 +301,17 @@ Session::Session(secure_vector<uint8_t>&& session_psk,
 
 Session::Session(std::string_view pem) : Session(PEM_Code::decode_check_label(pem, "TLS SESSION")) {}
 
-Session::Session(std::span<const uint8_t> ber_data) {
+Session::Session(std::span<const uint8_t> ber_data) /* NOLINT(*-member-init) */ {
    uint8_t side_code = 0;
 
+   std::vector<uint8_t> raw_pubkey_or_empty;
+
    ASN1_String server_hostname;
    ASN1_String server_service;
-   size_t server_port;
+   size_t server_port = 0;
 
-   uint8_t major_version = 0, minor_version = 0;
+   uint8_t major_version = 0;
+   uint8_t minor_version = 0;
 
    size_t start_time = 0;
    size_t srtp_profile = 0;
@@ -297,6 +331,7 @@ Session::Session(std::span<const uint8_t> ber_data) {
       .decode(m_encrypt_then_mac)
       .decode(m_master_secret, ASN1_Type::OctetString)
       .decode_list<X509_Certificate>(m_peer_certs)
+      .decode(raw_pubkey_or_empty, ASN1_Type::OctetString)
       .decode(server_hostname)
       .decode(server_service)
       .decode(server_port)
@@ -324,10 +359,17 @@ Session::Session(std::span<const uint8_t> ber_data) {
    m_server_info =
       Server_Information(server_hostname.value(), server_service.value(), static_cast<uint16_t>(server_port));
 
+   if(!raw_pubkey_or_empty.empty()) {
+      m_peer_raw_public_key = X509::load_key(raw_pubkey_or_empty);
+   }
+
    m_lifetime_hint = std::chrono::seconds(lifetime_hint);
 }
 
 secure_vector<uint8_t> Session::DER_encode() const {
+   const auto raw_pubkey_or_empty =
+      m_peer_raw_public_key ? m_peer_raw_public_key->subject_public_key() : std::vector<uint8_t>{};
+
    return DER_Encoder()
       .start_sequence()
       .encode(static_cast<size_t>(TLS_SESSION_PARAM_STRUCT_VERSION))
@@ -342,6 +384,7 @@ secure_vector<uint8_t> Session::DER_encode() const {
       .start_sequence()
       .encode_list(m_peer_certs)
       .end_cons()
+      .encode(raw_pubkey_or_empty, ASN1_Type::OctetString)
       .encode(ASN1_String(m_server_info.hostname(), ASN1_Type::Utf8String))
       .encode(ASN1_String(m_server_info.service(), ASN1_Type::Utf8String))
       .encode(static_cast<size_t>(m_server_info.port()))
@@ -411,7 +454,7 @@ std::vector<uint8_t> Session::encrypt(const SymmetricKey& key, RandomNumberGener
    std::vector<uint8_t> buf;
    buf.reserve(TLS_SESSION_CRYPT_OVERHEAD + bits.size());
    buf.resize(TLS_SESSION_CRYPT_MAGIC_LEN);
-   store_be(TLS_SESSION_CRYPT_MAGIC, &buf[0]);
+   store_be(TLS_SESSION_CRYPT_MAGIC, &buf[0]);  // NOLINT(*container-data-pointer)
    buf += key_name;
    buf += key_seed;
    buf += aead_nonce;
@@ -437,10 +480,10 @@ Session Session::decrypt(std::span<const uint8_t> in, const SymmetricKey& key) {
       }
 
       BufferSlicer sub(in);
-      const auto magic = sub.take(TLS_SESSION_CRYPT_MAGIC_LEN).data();
-      const auto key_name = sub.take(TLS_SESSION_CRYPT_KEY_NAME_LEN).data();
-      const auto key_seed = sub.take(TLS_SESSION_CRYPT_AEAD_KEY_SEED_LEN).data();
-      const auto aead_nonce = sub.take(TLS_SESSION_CRYPT_AEAD_NONCE_LEN).data();
+      const auto* const magic = sub.take(TLS_SESSION_CRYPT_MAGIC_LEN).data();
+      const auto* const key_name = sub.take(TLS_SESSION_CRYPT_KEY_NAME_LEN).data();
+      const auto* const key_seed = sub.take(TLS_SESSION_CRYPT_AEAD_KEY_SEED_LEN).data();
+      const auto* const aead_nonce = sub.take(TLS_SESSION_CRYPT_AEAD_NONCE_LEN).data();
       auto ctext = sub.copy_as_secure_vector(sub.remaining());
 
       if(load_be<uint64_t>(magic, 0) != TLS_SESSION_CRYPT_MAGIC) {
@@ -455,7 +498,7 @@ Session Session::decrypt(std::span<const uint8_t> in, const SymmetricKey& key) {
       hmac->update(TLS_SESSION_CRYPT_KEY_NAME);
       hmac->final(cmp_key_name.data());
 
-      if(same_mem(cmp_key_name.data(), key_name, TLS_SESSION_CRYPT_KEY_NAME_LEN) == false) {
+      if(CT::is_equal(cmp_key_name.data(), key_name, TLS_SESSION_CRYPT_KEY_NAME_LEN).as_bool() == false) {
          throw Decoding_Error("Wrong key name for encrypted session");
       }
 
