@@ -47,7 +47,7 @@ class BOTAN_TEST_API CPUFeature {
          HW_CLMUL = CLMUL,
       };
 
-      CPUFeature(Bit b) : m_bit(b) {}
+      CPUFeature(Bit b) : m_bit(b) {}  // NOLINT(*-explicit-conversions)
 
       uint32_t as_u32() const { return static_cast<uint32_t>(m_bit); }
 
