@@ -30,35 +30,53 @@ def get_concurrency():
 def known_targets():
     return [
         'amalgamation',
-        'bsi',
         'codeql',
         'coverage',
+        'cross-alpha',
         'cross-android-arm32',
         'cross-android-arm64',
+        'cross-android-arm64-amalgamation',
         'cross-arm32',
         'cross-arm32-baremetal',
         'cross-arm64',
+        'cross-arm64-amalgamation',
+        'cross-hppa64',
         'cross-i386',
         'cross-ios-arm64',
+        'cross-loongarch64',
+        'cross-m68k',
+        'cross-mips',
         'cross-mips64',
         'cross-ppc32',
         'cross-ppc64',
-        'cross-s390x',
         'cross-riscv64',
+        'cross-s390x',
+        'cross-sh4',
+        'cross-sparc64',
         'cross-win64',
         'docs',
         'emscripten',
         'examples',
         'format',
         'fuzzers',
+        'hybrid-tls13-interop-test',
         'lint',
+        'limbo',
         'minimized',
         'nist',
+        'no_pcurves',
+        'policy-bsi',
+        'policy-fips140',
+        'policy-modern',
         'sanitizer',
+        'sde',
         'shared',
         'static',
+        'strubbing',
         'valgrind',
         'valgrind-full',
+        'valgrind-ct',
+        'valgrind-ct-full',
     ]
 
 def is_running_in_github_actions():
@@ -72,6 +90,7 @@ class LoggingGroup:
 
     def __init__(self, group_title):
         self.group_title = group_title
+        self.start_time = time.time()
 
     def __enter__(self):
         if is_running_in_github_actions():
@@ -80,16 +99,20 @@ class LoggingGroup:
             print("Running '%s' ..." % self.group_title)
 
         sys.stdout.flush()
-        return is_running_in_github_actions()
 
     def __exit__(self, exc_type, exc_value, exc_tb):
+        time_taken = int(time.time() - self.start_time)
+
         if is_running_in_github_actions():
             print("::endgroup::")
 
+        if time_taken > 10:
+            print("> Running '%s' took %d seconds" % (self.group_title, time_taken))
+
 def build_targets(target, target_os):
-    if target in ['shared', 'minimized', 'bsi', 'nist']:
+    if target in ['shared', 'minimized', 'examples', 'limbo'] or target.startswith('policy-'):
         yield 'shared'
-    elif target in ['static', 'examples', 'fuzzers', 'cross-arm32-baremetal', 'emscripten']:
+    elif target in ['static', 'fuzzers', 'cross-arm32-baremetal', 'emscripten', 'strubbing']:
         yield 'static'
     elif target_os in ['windows']:
         yield 'shared'
@@ -99,20 +122,34 @@ def build_targets(target, target_os):
         yield 'shared'
         yield 'static'
 
-    if target not in ['examples']:
+    if target not in ['examples', 'limbo']:
         yield 'cli'
+
+    if target not in ['examples', 'limbo', 'hybrid-tls13-interop-test', 'strubbing']:
         yield 'tests'
 
     if target in ['coverage']:
         yield 'bogo_shim'
     if target in ['sanitizer'] and target_os not in ['windows']:
         yield 'bogo_shim'
-    if target in ['examples']:
+    if target in ['examples', 'amalgamation']:
         yield 'examples'
+    if target in ['valgrind', 'valgrind-full', 'valgrind-ct', 'valgrind-ct-full']:
+        yield 'ct_selftest'
+
+def make_targets(target, target_os):
+    # The result of build_targets() is for ./configure.py --build-targets='...'.
+    # make_targets() go into `make/ninja ...` and they are mostly equal. Except
+    # for 'libs' which is an umbrella target for 'static' and 'shared'.
+    tgts = [tgt if tgt not in ['static', 'shared'] else 'libs' for tgt in build_targets(target, target_os)]
+    if target in ['coverage', 'fuzzers']:
+        # These are special targets not found in ./configure.py --build-targets=
+        tgts += ['fuzzers', 'fuzzer_corpus_zip']
+    return list(set(tgts))
 
 def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                     root_dir, build_dir, test_results_dir, pkcs11_lib, use_gdb,
-                    disable_werror, extra_cxxflags, disabled_tests):
+                    disable_werror, extra_cxxflags, custom_optimization_flags, disabled_tests):
 
     """
     Return the configure.py flags as well as make/test running prefixes
@@ -128,7 +165,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
             target_os = 'ios'
         elif target == 'cross-win64':
             target_os = 'mingw'
-        elif target in ['cross-android-arm32', 'cross-android-arm64']:
+        elif target in ['cross-android-arm32', 'cross-android-arm64', 'cross-android-arm64-amalgamation']:
             target_os = 'android'
 
     if target_os == 'windows' and target_cc == 'gcc':
@@ -160,17 +197,20 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         test_cmd += ['--report-properties=%s' %
                      ','.join(['%s:%s' % (sanitize_kv(k), sanitize_kv(v)) for k, v in report_props.items()])]
 
-    install_prefix = tempfile.mkdtemp(prefix='botan-install-')
 
-    flags = ['--prefix=%s' % (install_prefix),
-             '--cc=%s' % (target_cc),
+    flags = ['--cc=%s' % (target_cc),
              '--os=%s' % (target_os),
              '--build-targets=%s' % ','.join(build_targets(target, target_os)),
              '--with-build-dir=%s' % build_dir,
-             '--link-method=symlink']
+             '--link-method=symlink',
+             '--enable-experimental-features']
+
+    if target in ['shared', 'static']:
+        install_prefix = tempfile.mkdtemp(prefix='botan-install-')
+        flags += ['--prefix=%s' % (install_prefix)]
 
     if ccache is not None:
-        flags += ['--no-store-vc-rev', '--compiler-cache=%s' % (ccache)]
+        flags += ['--compiler-cache=%s' % (ccache)]
 
     if not disable_werror:
         flags += ['--werror-mode']
@@ -178,25 +218,41 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target_cpu is not None:
         flags += ['--cpu=%s' % (target_cpu)]
 
+    if custom_optimization_flags and any(flag for flag in custom_optimization_flags):
+        flags += ['--no-optimizations']
+        for flag in custom_optimization_flags:
+            flags += ['--extra-cxxflags=%s' % (flag)]
+
     for flag in extra_cxxflags:
         flags += ['--extra-cxxflags=%s' % (flag)]
 
-    if target in ['minimized']:
-        flags += ['--minimized-build', '--enable-modules=system_rng,sha2_32,sha2_64,aes']
+    if target_os == 'windows':
+        # Workaround for https://github.com/actions/runner-images/issues/10004
+        flags += ['--extra-cxxflags=/D_DISABLE_CONSTEXPR_MUTEX_CONSTRUCTOR']
 
-    if target in ['amalgamation']:
+    if target in ['minimized']:
+        flags += ['--minimized-build', '--enable-modules=system_rng,sha2*,aes']
+
+    if target in ['no_pcurves']:
+        flags += ['--disable-modules=pcurves_impl']
+
+    if target in ['amalgamation', 'cross-arm64-amalgamation', 'cross-android-arm64-amalgamation']:
         flags += ['--amalgamation']
 
-    if target in ['bsi', 'nist']:
-        # tls is optional for bsi/nist but add it so verify tests work with these minimized configs
-        flags += ['--module-policy=%s' % (target), '--enable-modules=tls12']
+    if target.startswith('policy-'):
+        # tls is optional for bsi/fips140 but add it so verify tests work with these minimized configs
+        flags += ['--module-policy=%s' % (target.replace('policy-', '')), '--enable-modules=tls12,tls13', '--disable-deprecated-features']
 
-    if target == 'docs':
+    if target in ['docs']:
         flags += ['--with-doxygen', '--with-sphinx', '--with-rst2man']
+    else:
+        flags += ['--without-doc']
+
+    if target in ['docs', 'codeql', 'hybrid-tls13-interop-test', 'limbo']:
         test_cmd = None
 
-    if target == 'codeql':
-        test_cmd = None
+    if target in ['codeql']:
+        flags += ['--no-optimizations']
 
     if target == 'cross-win64':
         # this test compiles under MinGW but fails when run under Wine
@@ -209,32 +265,25 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target == 'coverage':
         flags += ['--with-coverage-info']
 
-    if target in ['coverage']:
+    if target in ['coverage', 'valgrind', 'valgrind-full', 'valgrind-ct', 'valgrind-ct-full']:
         flags += ['--with-debug-info']
 
+    if target in ['strubbing']:
+        # Stack scrubbing tests are based on scripted gdb runs and won't work on
+        # an optimized build unfortunately.
+        flags += ['--debug-mode']
+        test_cmd = None
+
     if target in ['coverage', 'sanitizer', 'fuzzers']:
-        flags += ['--terminate-on-asserts']
+        flags += ['--unsafe-terminate-on-asserts', '--enable-modules=tls_null']
 
-    if target in ['valgrind', 'valgrind-full']:
+    if target in ['sde']:
+        test_prefix = ['sde', '-future', '--']
+
+    if target in ['valgrind', 'valgrind-full', 'valgrind-ct', 'valgrind-ct-full']:
         flags += ['--with-valgrind']
-        test_prefix = ['valgrind', '--error-exitcode=9', '-v', '--leak-check=full', '--show-reachable=yes']
-        # valgrind is single threaded anyway
-        test_cmd += ['--test-threads=1']
-
-        if target != 'valgrind-full':
-            # valgrind is slow, so some tests only run in the nightly check
-            slow_tests = [
-                'argon2', 'bcrypt', 'bcrypt_pbkdf', 'compression_tests', 'cryptobox',
-                'dh_invalid', 'dh_kat', 'dh_keygen', 'dl_group_gen', 'dlies',
-                'dsa_kat_verify', 'dsa_param', 'ecc_basemul', 'ecdsa_verify_wycheproof',
-                'ed25519_sign', 'elgamal_decrypt', 'elgamal_encrypt', 'elgamal_keygen',
-                'ffi_dsa', 'ffi_elgamal', 'hash_nist_mc', 'mce_keygen',
-                'passhash9', 'pbkdf', 'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw',
-                'scrypt', 'srp6_kat', 'srp6_rt', 'unit_tls', 'x509_path_bsi',
-                'x509_path_rsa_pss', 'xmss_keygen', 'xmss_keygen_reference',
-                'xmss_sign', 'xmss_verify', 'xmss_verify_invalid' ]
-
-            disabled_tests += slow_tests
+        # valgrind is run via a script setup later
+        test_cmd = None
 
     if target == 'examples':
         flags += ['--with-boost']
@@ -249,12 +298,12 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target in ['fuzzers', 'sanitizer']:
         flags += ['--with-debug-asserts']
 
-        if target_cc in ['clang', 'gcc']:
-            flags += ['--enable-sanitizers=address,undefined']
+        if target_cc in ['clang', 'gcc', 'xcode']:
+            flags += ['--enable-sanitizers=address,undefined,iterator']
         else:
-            flags += ['--enable-sanitizers=address']
+            flags += ['--enable-sanitizers=address,iterator']
 
-    if target in ['valgrind', 'valgrind-full', 'sanitizer', 'fuzzers']:
+    if target in ['valgrind', 'valgrind-full', 'valgrind-ct', 'valgrind-ct-full', 'sanitizer', 'fuzzers']:
         flags += ['--disable-modules=locking_allocator']
 
     if target == 'emscripten':
@@ -262,6 +311,13 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         # need to find a way to run the wasm-compiled tests w/o a browser
         test_cmd = None
 
+    if target in ['sanitizer', 'strubbing'] and target_cc in ['gcc']:
+        # Stack scrubbing is supported on GCC 14 and newer, only. This is newer
+        # than the current default compiler on GHA's Linux image (ubuntu 24.04).
+        # The CI setup has to ensure that we are configured to use a recent
+        # compiler for these targets, otherwise `./configure.py` will fail.
+        flags += ['--enable-stack-scrubbing']
+
     if is_cross_target:
         if target_os == 'ios':
             make_prefix = ['xcrun', '--sdk', 'iphoneos']
@@ -288,7 +344,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 cc_bin = os.path.join(toolchain_dir, 'armv7a-linux-androideabi%d-clang++' % (api_lvl))
                 flags += ['--cpu=armv7',
                           '--ar-command=%s' % (os.path.join(toolchain_dir, 'llvm-ar'))]
-            elif target == 'cross-android-arm64':
+            elif target in ['cross-android-arm64', 'cross-android-arm64-amalgamation']:
                 cc_bin = os.path.join(toolchain_dir, 'aarch64-linux-android%d-clang++' % (api_lvl))
                 flags += ['--cpu=arm64',
                           '--ar-command=%s' % (os.path.join(toolchain_dir, 'llvm-ar'))]
@@ -313,18 +369,38 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 flags += ['--cpu=armv7', '--extra-cxxflags=-D_FILE_OFFSET_BITS=64']
                 cc_bin = 'arm-linux-gnueabihf-g++'
                 test_prefix = ['qemu-arm', '-L', '/usr/arm-linux-gnueabihf/']
-                # disable a few tests that are exceptionally slow under arm32 qemu
-                disabled_tests += ['dh_invalid', 'dlies', 'xmss_sign']
-            elif target == 'cross-arm64':
+            elif target in ['cross-arm64', 'cross-arm64-amalgamation']:
                 flags += ['--cpu=aarch64']
                 cc_bin = 'aarch64-linux-gnu-g++'
                 test_prefix = ['qemu-aarch64', '-L', '/usr/aarch64-linux-gnu/']
+            elif target == 'cross-alpha':
+                flags += ['--cpu=alpha']
+                cc_bin = 'alpha-linux-gnu-g++'
+                test_prefix = ['qemu-alpha', '-L', '/usr/alpha-linux-gnu/']
+                flags += ['--without-stack-protector'] # not supported
+            elif target == 'cross-sh4':
+                flags += ['--cpu=sh4']
+                cc_bin = 'sh4-linux-gnu-g++'
+                test_prefix = ['qemu-sh4', '-L', '/usr/sh4-linux-gnu/']
+            elif target == 'cross-m68k':
+                flags += ['--cpu=m68k']
+                cc_bin = 'm68k-linux-gnu-g++'
+                test_prefix = ['qemu-m68k', '-L', '/usr/m68k-linux-gnu/']
+            elif target == 'cross-hppa64':
+                flags += ['--cpu=hppa']
+                cc_bin = 'hppa-linux-gnu-g++'
+                test_prefix = ['qemu-hppa', '-L', '/usr/hppa-linux-gnu/']
+            elif target == 'cross-sparc64':
+                flags += ['--cpu=sparc64']
+                cc_bin = 'sparc64-linux-gnu-g++'
+                test_prefix = ['qemu-sparc64', '-L', '/usr/sparc64-linux-gnu/']
             elif target == 'cross-ppc32':
                 flags += ['--cpu=ppc32']
                 cc_bin = 'powerpc-linux-gnu-g++'
                 test_prefix = ['qemu-ppc', '-L', '/usr/powerpc-linux-gnu/']
+                test_cmd = None # qemu crashes ...
             elif target == 'cross-ppc64':
-                flags += ['--cpu=ppc64', '--with-endian=little']
+                flags += ['--cpu=ppc64']
                 cc_bin = 'powerpc64le-linux-gnu-g++'
                 test_prefix = ['qemu-ppc64le', '-cpu', 'power10', '-L', '/usr/powerpc64le-linux-gnu/']
             elif target == 'cross-riscv64':
@@ -335,13 +411,20 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 flags += ['--cpu=s390x']
                 cc_bin = 's390x-linux-gnu-g++'
                 test_prefix = ['qemu-s390x', '-L', '/usr/s390x-linux-gnu/']
+            elif target == 'cross-loongarch64':
+                flags += ['--cpu=loongarch64']
+                cc_bin = 'loongarch64-linux-gnu-g++-14'
+                test_prefix = ['qemu-loongarch64', '-L', '/usr/loongarch64-linux-gnu/']
+            elif target == 'cross-mips':
+                flags += ['--cpu=mips32']
+                cc_bin = 'mips-linux-gnu-g++'
+                test_prefix = ['qemu-mips', '-L', '/usr/mips-linux-gnu/']
             elif target == 'cross-mips64':
-                flags += ['--cpu=mips64', '--with-endian=big']
+                flags += ['--cpu=mips64']
                 cc_bin = 'mips64-linux-gnuabi64-g++'
                 test_prefix = ['qemu-mips64', '-L', '/usr/mips64-linux-gnuabi64/']
-                test_cmd.remove('simd_32') # no SIMD on MIPS
             elif target in ['cross-arm32-baremetal']:
-                flags += ['--cpu=arm32', '--disable-neon', '--without-stack-protector', '--ldflags=-specs=nosys.specs']
+                flags += ['--cpu=arm32', '--without-stack-protector', '--ldflags=-specs=nosys.specs', '--disable-modules=simd_4x32']
                 cc_bin = 'arm-none-eabi-c++'
                 test_cmd = None
             else:
@@ -356,24 +439,27 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
             flags += ['--with-commoncrypto']
 
         def add_boost_support(target, target_os):
-            if target in ['coverage', 'shared']:
+            if target in ['coverage', 'amalgamation']:
                 return True
 
             if target == 'sanitizer' and target_os == 'linux':
                 return True
 
+            if target == 'shared' and target_os != 'windows':
+                return True
+
             return False
 
         if add_boost_support(target, target_os):
             flags += ['--with-boost']
-            if target_cc == 'clang':
+            if target_cc in ['clang', 'xcode']:
                 # make sure clang ignores warnings in boost headers
                 flags += ["--extra-cxxflags=--system-header-prefix=boost/"]
 
-            if target_os in ['windows', 'mingw']:
-                # ./configure.py needs boost's location on Windows
-                if 'BOOST_INCLUDEDIR' in os.environ:
-                    flags += ['--with-external-includedir', os.environ.get('BOOST_INCLUDEDIR')]
+            if 'BOOST_INCLUDEDIR' in os.environ:
+                # ./configure.py needs boost's location on some platforms
+                # BOOST_INCLUDEDIR is set by the setup_gh_actions.* script
+                flags += ['--with-external-includedir', os.environ.get('BOOST_INCLUDEDIR')]
 
             if target_os == 'mingw':
                 # apparently mingw needs this legacy socket library version for reasons
@@ -383,6 +469,31 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         if target_os == 'linux':
             flags += ['--with-lzma']
 
+        if is_running_in_github_actions() and 'BOTAN_TPM2_ENABLED' in os.environ:
+            flags += ['--with-tpm2']
+
+            if os.environ.get('BOTAN_TPM2_ENABLED') == 'test':
+                # run the TPM2 tests
+                test_cmd += ["--tpm2-tcti-name=%s" % os.getenv('BOTAN_TPM2_TCTI_NAME'),
+                             "--tpm2-tcti-conf=%s" % os.getenv('BOTAN_TPM2_TCTI_CONF'),
+                             "--tpm2-persistent-rsa-handle=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_RSA_KEY_HANDLE'),
+                             "--tpm2-persistent-ecc-handle=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_ECC_KEY_HANDLE'),
+                             "--tpm2-persistent-auth-value=%s" % os.getenv('BOTAN_TPM2_PERSISTENT_KEY_AUTH_VALUE')]
+            elif os.environ.get('BOTAN_TPM2_ENABLED') == 'build':
+                # build the TPM2 module but don't run the tests
+                # TCTI name 'disabled' is a special value that disables the TPM2 tests
+                #
+                # TODO: This is a hack. It would be great if `./botan-test --skip-tests=`
+                #       would also work for test categories like `tpm2`. Currently it
+                #       only works for individual test names.
+                test_cmd += ["--tpm2-tcti-name=disabled"]
+
+        if is_running_in_github_actions():
+            if 'BOTAN_BUILD_WITH_JITTERENTROPY' in os.environ:
+                flags += ['--enable-modules=jitter_rng']
+            if 'BOTAN_BUILD_WITH_ESDM' in os.environ:
+                flags += ['--with-esdm_rng']
+
         if target in ['coverage']:
             flags += ['--with-tpm']
             test_cmd += ['--run-online-tests']
@@ -401,7 +512,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
             # slower tests to take as long as 5 minutes
             test_cmd.remove('--run-long-tests')
 
-    flags += ['--cc-bin=%s' % (cc_bin)]
+    if os.getenv('CXX') is None:
+        flags += ['--cc-bin=%s' % (cc_bin)]
 
     if test_cmd is None:
         run_test_command = None
@@ -430,8 +542,6 @@ def run_cmd(cmd, root_dir, build_dir):
     """
 
     with LoggingGroup(' '.join(cmd)):
-        start = time.time()
-
         cmd = [os.path.expandvars(elem) for elem in cmd]
         sub_env = os.environ.copy()
         sub_env['LD_LIBRARY_PATH'] = os.path.abspath(build_dir)
@@ -457,11 +567,6 @@ def run_cmd(cmd, root_dir, build_dir):
         proc = subprocess.Popen(cmd, cwd=cwd, close_fds=True, env=sub_env, stdout=redirect_stdout_fd)
         proc.communicate()
 
-        time_taken = int(time.time() - start)
-
-        if time_taken > 10:
-            print("Ran for %d seconds" % (time_taken))
-
         if proc.returncode != 0:
             print("Command '%s' failed with error code %d" % (' '.join(cmd), proc.returncode))
 
@@ -479,6 +584,14 @@ def default_os():
         return 'osx'
     return platform_os
 
+def default_cc():
+    platform_os = platform.system().lower()
+    return 'msvc' if platform_os == 'windows' else 'gcc'
+
+def default_make_tool():
+    platform_os = platform.system().lower()
+    return 'nmake' if platform_os == 'windows' else 'make'
+
 def parse_args(args):
     """
     Parse arguments
@@ -489,7 +602,7 @@ def parse_args(args):
                       help='Set the target os (default %default)')
     parser.add_option('--cpu', default=None,
                       help='Specify a target CPU platform')
-    parser.add_option('--cc', default='gcc',
+    parser.add_option('--cc', default=default_cc(),
                       help='Set the target compiler type (default %default)')
     parser.add_option('--cc-bin', default=None,
                       help='Set path to compiler')
@@ -500,11 +613,13 @@ def parse_args(args):
     parser.add_option('--boringssl-dir', metavar='D', default='boringssl',
                       help='Set directory of BoringSSL checkout to use for BoGo tests')
 
-    parser.add_option('--make-tool', metavar='TOOL', default='make',
+    parser.add_option('--make-tool', metavar='TOOL', default=default_make_tool(),
                       help='Specify tool to run to build source (default %default)')
 
     parser.add_option('--extra-cxxflags', metavar='FLAGS', default=[], action='append',
                       help='Specify extra build flags')
+    parser.add_option('--custom-optimization-flags', metavar='FLAGS', default=[], action='append',
+                      help='Specify custom optimization flags (disables all default optimizations)')
 
     parser.add_option('--disabled-tests', metavar='DISABLED_TESTS', default=[], action='append',
                       help='Comma separated list of tests that should not be run')
@@ -515,7 +630,7 @@ def parse_args(args):
                       help='Set number of jobs to run in parallel (default %default)')
 
     parser.add_option('--compiler-cache', default=None, metavar='CC',
-                      help='Set a compiler cache to use (ccache, sccache)')
+                      help='Set a compiler cache to use (ccache, sccache, none)')
 
     parser.add_option('--pkcs11-lib', default=os.getenv('PKCS11_LIB'), metavar='LIB',
                       help='Set PKCS11 lib to use for testing')
@@ -551,7 +666,8 @@ def validate_make_tool(make_tool, build_jobs):
 
     if make_tool in ['make']:
         return make_tool, ['-j%d' % (build_jobs), '-k']
-
+    elif make_tool in ['ninja']:
+        return make_tool, ['-k', '0']
     else:
         return make_tool, []
 
@@ -584,8 +700,13 @@ def main(args=None):
     if options.cc_bin is None:
         if options.cc == 'gcc':
             options.cc_bin = 'g++'
+        elif options.cc == 'gcc-14':
+            options.cc = 'gcc' # Hack: 'gcc-14' is not a valid compiler identifier for ``./configure.py --cc``
+            options.cc_bin = 'g++-14'
         elif options.cc == 'clang':
             options.cc_bin = 'clang++'
+        elif options.cc == 'xcode':
+            options.cc_bin = 'clang++'
         elif options.cc == 'msvc':
             options.cc_bin = 'cl'
         elif options.cc == "emcc":
@@ -603,6 +724,9 @@ def main(args=None):
         if options.compiler_cache:
             print("Found '%s' installed, will use it..." % (options.compiler_cache))
 
+    if options.compiler_cache == 'none':
+        options.compiler_cache = None
+
     if options.compiler_cache not in [None, 'ccache', 'sccache']:
         raise Exception("Don't know about %s as a compiler cache" % (options.compiler_cache))
 
@@ -622,6 +746,10 @@ def main(args=None):
 
         pylint_rc = '--rcfile=%s' % (os.path.join(root_dir, 'src/configs/pylint.rc'))
         pylint_flags = [pylint_rc, '--reports=no']
+        pylint_flags += ['--ignored-modules=gdb'] # 'import gdb' is not available outside gdb...
+
+        if is_running_in_github_actions():
+            pylint_flags += ["--msg-template='::warning file={path},line={line},endLine={end_line}::Pylint ({category}): {msg_id} {msg} ({symbol})'"]
 
         py_scripts = [
             'configure.py',
@@ -633,27 +761,34 @@ def main(args=None):
             'src/scripts/dist.py',
             'src/scripts/cleanup.py',
             'src/scripts/check.py',
+            'src/scripts/compare_perf.py',
             'src/scripts/build_docs.py',
             'src/scripts/website.py',
             'src/scripts/bench.py',
             'src/scripts/test_python.py',
+            'src/scripts/test_strubbed_symbols.py',
             'src/scripts/test_fuzzers.py',
             'src/scripts/test_cli.py',
+            'src/scripts/repo_config.py',
             'src/scripts/python_unittests.py',
             'src/scripts/python_unittests_unix.py',
             'src/scripts/dev_tools/run_clang_format.py',
             'src/scripts/dev_tools/run_clang_tidy.py',
+            'src/scripts/gdb/strubtest.py',
             'src/editors/vscode/scripts/bogo.py',
             'src/editors/vscode/scripts/common.py',
-            'src/editors/vscode/scripts/test.py']
+            'src/editors/vscode/scripts/test.py',
+            'src/ct_selftest/ct_selftest.py']
 
-        full_paths = [os.path.join(root_dir, s) for s in py_scripts]
-        cmds.append([py_interp, '-m', 'pylint'] + pylint_flags + full_paths)
+        # This has to run in the repository root to generate the correct
+        # relative paths in the output. Otherwise GitHub Actions will not
+        # be able to annotate the correct files.
+        cmds.append(["indir:%s" % root_dir, py_interp, '-m', 'pylint'] + pylint_flags + py_scripts)
 
     elif target == 'format':
         cmds.append([py_interp,
                      os.path.join(root_dir, 'src/scripts/dev_tools/run_clang_format.py'),
-                     '--clang-format=clang-format-15',
+                     '--clang-format=clang-format-17',
                      '--src-dir=%s' % (os.path.join(root_dir, 'src')),
                      '--check'])
     else:
@@ -664,7 +799,7 @@ def main(args=None):
             target, options.os, options.cpu, options.cc, options.cc_bin,
             options.compiler_cache, root_dir, build_dir, options.test_results_dir,
             options.pkcs11_lib, options.use_gdb, options.disable_werror,
-            options.extra_cxxflags, options.disabled_tests)
+            options.extra_cxxflags, options.custom_optimization_flags, options.disabled_tests)
 
         make_tool, make_opts = validate_make_tool(options.make_tool, options.build_jobs)
 
@@ -684,18 +819,14 @@ def main(args=None):
             if options.compiler_cache is not None:
                 cmds.append([options.compiler_cache, '--show-stats'])
 
-            make_targets = ['libs', 'tests', 'cli']
+            cmds.append(make_prefix + make_cmd + make_targets(target, options.os))
 
-            if target in ['coverage', 'fuzzers']:
-                make_targets += ['fuzzer_corpus_zip', 'fuzzers']
-
-            if target in ['examples']:
-                make_targets += ['examples']
-
-            if target in ['coverage', 'sanitizer'] and options.os not in ['windows']:
-                make_targets += ['bogo_shim']
-
-            cmds.append(make_prefix + make_cmd + make_targets)
+            if target in ['examples'] and options.cc in ['clang', 'gcc']:
+                cmds.append([options.cc, '-Wall', '-Wextra', '-std=c89',
+                             '-I%s' % (os.path.join(build_dir, 'build/include/public')),
+                             os.path.join(root_dir, 'src/examples/ffi.c'),
+                             '-L%s' % (build_dir), '-lbotan-3', '-o',
+                             os.path.join(build_dir, 'build/examples/ffi')])
 
             if options.compiler_cache is not None:
                 cmds.append([options.compiler_cache, '--show-stats'])
@@ -703,6 +834,53 @@ def main(args=None):
         if run_test_command is not None:
             cmds.append(run_test_command)
 
+        if target in ['valgrind', 'valgrind-full', 'valgrind-ct', 'valgrind-ct-full']:
+
+            build_config = os.path.join(build_dir, 'build', 'build_config.json')
+            cmds.append([os.path.join(root_dir, 'src', 'ct_selftest', 'ct_selftest.py'),
+                         "--build-config-path=%s" % build_config,
+                         os.path.join(build_dir, 'botan_ct_selftest')])
+
+            valgrind_script_options = ['--test-binary=%s' % (os.path.join(build_dir, 'botan-test')),
+                                       '--verbose',
+                                       '--bunch',
+                                       '--track-origins']
+
+            # For finding memory bugs, we're enabling more features that add runtime
+            # overhead which we don't need for the secret-dependent execution checks
+            # that 'valgrind-ct' and 'valgrind-ct-full' are aiming for.
+            if target not in ['valgrind-ct', 'valgrind-ct-full']:
+                valgrind_script_options.append('--with-leak-check')
+
+            if target not in ['valgrind-full', 'valgrind-ct-full']:
+                # valgrind is slow, so some tests only run in the nightly check
+                slow_tests = [
+                    'argon2', 'bcrypt', 'bcrypt_pbkdf', 'compression_tests', 'cryptobox',
+                    'dh_invalid', 'dh_kat', 'dh_keygen', 'dl_group_gen', 'dlies',
+                    'dsa_kat_verify', 'dsa_param', 'ecc_basemul', 'ecdsa_verify_wycheproof',
+                    'ed25519_sign', 'elgamal_decrypt', 'elgamal_encrypt', 'elgamal_keygen',
+                    'ffi_dh', 'ffi_dsa', 'ffi_elgamal', 'frodo_kat_tests', 'hash_nist_mc',
+                    'hss_lms_keygen', 'hss_lms_sign', 'mce_keygen', 'passhash9', 'pbkdf',
+                    'pcurves_arith', 'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw', 'scrypt',
+                    'sphincsplus', 'sphincsplus_fors', 'slh_dsa_keygen', 'slh_dsa', 'srp6_kat',
+                    'srp6_rt', 'unit_tls', 'x509_path_bsi', 'x509_path_rsa_pss',
+                    'xmss_keygen', 'xmss_keygen_reference', 'xmss_sign', 'xmss_unit_tests',
+                    'xmss_verify', 'xmss_verify_invalid',
+                ]
+                slow_tests += [f"dilithium_kat_{mode}_{rand}" for mode in ('6x5', '8x7', '6x5_AES', '8x7_AES') for rand in ('Deterministic', 'Randomized')]
+                slow_tests += [f"ml_dsa_kat_{mode}_{rand}" for mode in ('6x5', '8x7') for rand in ('Deterministic', 'Randomized')]
+
+                valgrind_script_options.append('--skip-tests=%s' % (','.join(slow_tests)))
+            elif target == 'valgrind-ct-full' and options.cc == 'clang' and '-Os' in options.custom_optimization_flags:
+                # Clang 18 (only) with -Os seems to have a problem with std::optional which flags certain
+                # uses as touching an unitialized stack variable. This affects the x509_rpki tests
+                # TODO(26.04) We can remove this once we have a new version of Clang to use
+                valgrind_script_options.append('--skip-tests=x509_rpki')
+
+            cmds.append(['indir:%s' % (root_dir),
+                         'src/scripts/run_tests_under_valgrind.py'] +
+                        valgrind_script_options)
+
         if target in ['coverage', 'sanitizer'] and options.os != 'windows':
             if not options.boringssl_dir:
                 raise Exception('coverage build needs --boringssl-dir')
@@ -715,12 +893,16 @@ def main(args=None):
                          '-shim-path', os.path.abspath(os.path.join(build_dir, 'botan_bogo_shim')),
                          '-shim-config', os.path.abspath(os.path.join(root_dir, 'src', 'bogo_shim', 'config.json'))])
 
+        if target in ['limbo']:
+            cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/run_limbo_tests.py'),
+                         os.path.join(root_dir, 'limbo.json')])
+
         if target in ['coverage', 'fuzzers']:
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/test_fuzzers.py'),
                          os.path.join(build_dir, 'fuzzer_corpus'),
                          os.path.join(build_dir, 'build/fuzzer')])
 
-        if target in ['shared', 'coverage', 'sanitizer'] and options.os != 'windows':
+        if target in ['shared', 'coverage', 'sanitizer']:
             botan_exe = os.path.join(build_dir, 'botan-cli.exe' if options.os == 'windows' else 'botan')
 
             args = ['--threads=%d' % (options.build_jobs)]
@@ -734,18 +916,31 @@ def main(args=None):
                 cmds.append([py_interp, os.path.join(root_dir, 'src/scripts', script)] +
                             args + test_data_arg + [botan_exe])
 
+        if target in ['strubbing']:
+            cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/test_strubbed_symbols.py'),
+                         '--botan-cli', os.path.join(build_dir, 'botan')])
+
+        if target in ['hybrid-tls13-interop-test']:
+            cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/test_cli.py'),
+                         '--run-online-tests', os.path.join(build_dir, 'botan'), 'pqc_hybrid_tests'])
+
         python_tests = [os.path.join(root_dir, 'src/scripts/test_python.py')]
         if root_dir != '.':
             python_tests.append('--test-data-dir=%s' % root_dir)
 
+        if is_running_in_github_actions() and os.environ.get('BOTAN_TPM2_ENABLED', 'no') == 'test':
+            python_tests.extend(["--tpm2-tcti-name=%s" % os.getenv('BOTAN_TPM2_TCTI_NAME'),
+                                 "--tpm2-tcti-conf=%s" % os.getenv('BOTAN_TPM2_TCTI_CONF')])
+
         if target in ['shared', 'coverage'] and not (options.os == 'windows' and options.cpu == 'x86'):
             cmds.append([py_interp, '-b'] + python_tests)
 
-        if target in ['shared', 'static', 'bsi', 'nist']:
+        if target in ['shared', 'static']:
             cmds.append(make_cmd + ['install'])
             build_config = os.path.join(build_dir, 'build', 'build_config.json')
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/ci_check_install.py'), build_config])
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/ci_check_headers.py'), build_config])
+            cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/ci_report_sizes.py'), build_config])
 
         if target in ['coverage']:
             if have_prog('coverage'):
@@ -764,7 +959,7 @@ def main(args=None):
 
             if have_prog('coveralls'):
                 # If coveralls command exists, assume we are in CI and report to coveralls.io
-                cmds.append(['coveralls', '--format=lcov', '--file=%s' % (cov_file)])
+                cmds.append(['coveralls', '--no-fail', '--format=lcov', '--file=%s' % (cov_file)])
             else:
                 # Otherwise generate a local HTML report
                 cmds.append(['genhtml', cov_file, '--output-directory', os.path.join(build_dir, 'lcov-out')])
@@ -772,12 +967,23 @@ def main(args=None):
         cmds.append(make_cmd + ['clean'])
         cmds.append(make_cmd + ['distclean'])
 
+    esdm_process = None
+    # start ESDM in background, if on Linux
+    if is_running_in_github_actions() and 'BOTAN_BUILD_WITH_ESDM' in os.environ:
+        print('Starting esdm-server for this target')
+        esdm_process = subprocess.Popen('sudo /usr/bin/esdm-server -f', shell=True)
+        assert esdm_process.poll() is None, f"esdm-server did not start for target {target}"
+
     for cmd in cmds:
         if options.dry_run:
             print('$ ' + ' '.join(cmd))
         else:
             run_cmd(cmd, root_dir, build_dir)
 
+    if esdm_process:
+        print('Stopping esdm-server')
+        esdm_process.kill()
+
     return 0
 
 if __name__ == '__main__':
