@@ -9,6 +9,7 @@
 
 #if defined(BOTAN_HAS_AEAD_MODES)
    #include <botan/aead.h>
+   #include <botan/mem_ops.h>
 #endif
 
 namespace Botan_Tests {
@@ -26,7 +27,8 @@ class AEAD_Tests final : public Text_Based_Test {
                                    const std::vector<uint8_t>& input,
                                    const std::vector<uint8_t>& expected,
                                    const std::vector<uint8_t>& ad,
-                                   const std::string& algo) {
+                                   const std::string& algo,
+                                   Botan::RandomNumberGenerator& rng) {
          const bool is_siv = algo.find("/SIV") != std::string::npos;
 
          Test::Result result(algo);
@@ -38,17 +40,23 @@ class AEAD_Tests final : public Text_Based_Test {
          result.confirm("AEAD name is not empty", !enc->name().empty());
          result.confirm("AEAD default nonce size is accepted", enc->valid_nonce_length(enc->default_nonce_length()));
 
-         Botan::secure_vector<uint8_t> garbage = Test::rng().random_vec(enc->update_granularity());
+         auto get_garbage = [&] { return rng.random_vec(enc->update_granularity()); };
 
-         if(is_siv == false) {
-            result.test_throws("Unkeyed object throws for encrypt", [&]() { enc->update(garbage); });
+         if(!is_siv) {
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for encrypt", [&]() {
+               auto garbage = get_garbage();
+               enc->update(garbage);
+            });
          }
 
-         result.test_throws("Unkeyed object throws for encrypt", [&]() { enc->finish(garbage); });
+         result.test_throws<Botan::Invalid_State>("Unkeyed object throws for encrypt", [&]() {
+            auto garbage = get_garbage();
+            enc->finish(garbage);
+         });
 
          if(enc->associated_data_requires_key()) {
-            result.test_throws("Unkeyed object throws for set AD",
-                               [&]() { enc->set_associated_data(ad.data(), ad.size()); });
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for set AD",
+                                                     [&]() { enc->set_associated_data(ad.data(), ad.size()); });
          }
 
          result.test_eq("key is not set", enc->has_keying_material(), false);
@@ -58,13 +66,21 @@ class AEAD_Tests final : public Text_Based_Test {
          enc->set_key(key);
          result.test_eq("key is set", enc->has_keying_material(), true);
 
-         if(is_siv == false) {
-            result.test_throws("Cannot process data until nonce is set (enc)", [&]() { enc->update(garbage); });
-            result.test_throws("Cannot process data until nonce is set (enc)", [&]() { enc->finish(garbage); });
+         if(!is_siv) {
+            result.test_throws<Botan::Invalid_State>("Cannot process data until nonce is set (enc)", [&]() {
+               auto garbage = get_garbage();
+               enc->update(garbage);
+            });
+            result.test_throws<Botan::Invalid_State>("Cannot process data until nonce is set (enc)", [&]() {
+               auto garbage = get_garbage();
+               enc->finish(garbage);
+            });
          }
 
-         enc->set_associated_data(mutate_vec(ad));
-         enc->start(mutate_vec(nonce));
+         enc->set_associated_data(mutate_vec(ad, rng));
+         enc->start(mutate_vec(nonce, rng));
+
+         auto garbage = get_garbage();
          enc->update(garbage);
 
          // reset message specific state
@@ -107,24 +123,24 @@ class AEAD_Tests final : public Text_Based_Test {
 
                buf.assign(input.begin(), input.end());
                size_t input_length = buf.size();
-               size_t offset = 0;
                uint8_t* p = buf.data();
                Botan::secure_vector<uint8_t> block(update_granularity);
-               Botan::secure_vector<uint8_t> ciphertext(enc->output_length(buf.size()));
+               Botan::secure_vector<uint8_t> ciphertext;
+               ciphertext.reserve(enc->output_length(buf.size()));
                while(input_length > update_granularity &&
                      ((input_length - update_granularity) >= enc->minimum_final_size())) {
                   block.assign(p, p + update_granularity);
                   enc->update(block);
                   p += update_granularity;
                   input_length -= update_granularity;
-                  buffer_insert(ciphertext, 0 + offset, block);
-                  offset += block.size();
+
+                  ciphertext.insert(ciphertext.end(), block.begin(), block.end());
                }
 
                // encrypt remaining bytes
                block.assign(p, p + input_length);
                enc->finish(block);
-               buffer_insert(ciphertext, 0 + offset, block);
+               ciphertext.insert(ciphertext.end(), block.begin(), block.end());
 
                result.test_eq("encrypt update", ciphertext, expected);
             }
@@ -168,11 +184,12 @@ class AEAD_Tests final : public Text_Based_Test {
          enc->clear();
          result.test_eq("key is not set", enc->has_keying_material(), false);
 
-         result.test_throws("Unkeyed object throws for encrypt after clear", [&]() { enc->finish(buf); });
+         result.test_throws<Botan::Invalid_State>("Unkeyed object throws for encrypt after clear",
+                                                  [&]() { enc->finish(buf); });
 
          if(enc->associated_data_requires_key()) {
-            result.test_throws("Unkeyed object throws for set AD after clear",
-                               [&]() { enc->set_associated_data(ad.data(), ad.size()); });
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for set AD after clear",
+                                                     [&]() { enc->set_associated_data(ad.data(), ad.size()); });
          }
 
          return result;
@@ -183,7 +200,8 @@ class AEAD_Tests final : public Text_Based_Test {
                                    const std::vector<uint8_t>& input,
                                    const std::vector<uint8_t>& expected,
                                    const std::vector<uint8_t>& ad,
-                                   const std::string& algo) {
+                                   const std::string& algo,
+                                   Botan::RandomNumberGenerator& rng) {
          const bool is_siv = algo.find("/SIV") != std::string::npos;
 
          Test::Result result(algo);
@@ -192,17 +210,24 @@ class AEAD_Tests final : public Text_Based_Test {
 
          result.test_eq("AEAD decrypt output_length is correct", dec->output_length(input.size()), expected.size());
 
-         Botan::secure_vector<uint8_t> garbage = Test::rng().random_vec(dec->update_granularity());
+         auto get_garbage = [&] { return rng.random_vec(dec->update_granularity()); };
+         auto get_ultimate_garbage = [&] { return rng.random_vec(dec->minimum_final_size()); };
 
-         if(is_siv == false) {
-            result.test_throws("Unkeyed object throws for decrypt", [&]() { dec->update(garbage); });
+         if(!is_siv) {
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for decrypt", [&]() {
+               auto garbage = get_garbage();
+               dec->update(garbage);
+            });
          }
 
-         result.test_throws("Unkeyed object throws for decrypt", [&]() { dec->finish(garbage); });
+         result.test_throws<Botan::Invalid_State>("Unkeyed object throws for decrypt", [&]() {
+            auto garbage = get_ultimate_garbage();
+            dec->finish(garbage);
+         });
 
          if(dec->associated_data_requires_key()) {
-            result.test_throws("Unkeyed object throws for set AD",
-                               [&]() { dec->set_associated_data(ad.data(), ad.size()); });
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for set AD",
+                                                     [&]() { dec->set_associated_data(ad.data(), ad.size()); });
          }
 
          // First some tests for reset() to make sure it resets what we need it to
@@ -210,15 +235,21 @@ class AEAD_Tests final : public Text_Based_Test {
          result.test_eq("key is not set", dec->has_keying_material(), false);
          dec->set_key(key);
          result.test_eq("key is set", dec->has_keying_material(), true);
-         dec->set_associated_data(mutate_vec(ad));
+         dec->set_associated_data(mutate_vec(ad, rng));
 
-         if(is_siv == false) {
-            result.test_throws("Cannot process data until nonce is set (dec)", [&]() { dec->update(garbage); });
-            result.test_throws("Cannot process data until nonce is set (dec)", [&]() { dec->finish(garbage); });
+         if(!is_siv) {
+            result.test_throws<Botan::Invalid_State>("Cannot process data until nonce is set (dec)", [&]() {
+               auto garbage = get_garbage();
+               dec->update(garbage);
+            });
+            result.test_throws<Botan::Invalid_State>("Cannot process data until nonce is set (dec)", [&]() {
+               auto garbage = get_ultimate_garbage();
+               dec->finish(garbage);
+            });
          }
 
-         dec->start(mutate_vec(nonce));
-
+         dec->start(mutate_vec(nonce, rng));
+         auto garbage = get_garbage();
          dec->update(garbage);
 
          // reset message specific state
@@ -253,24 +284,23 @@ class AEAD_Tests final : public Text_Based_Test {
 
                buf.assign(input.begin(), input.end());
                size_t input_length = buf.size();
-               size_t offset = 0;
                uint8_t* p = buf.data();
                Botan::secure_vector<uint8_t> block(update_granularity);
-               Botan::secure_vector<uint8_t> plaintext(dec->output_length(buf.size()));
+               Botan::secure_vector<uint8_t> plaintext;
+               plaintext.reserve(dec->output_length(buf.size()));
                while((input_length > update_granularity) &&
                      ((input_length - update_granularity) >= dec->minimum_final_size())) {
                   block.assign(p, p + update_granularity);
                   dec->update(block);
                   p += update_granularity;
                   input_length -= update_granularity;
-                  buffer_insert(plaintext, 0 + offset, block);
-                  offset += block.size();
+                  plaintext.insert(plaintext.end(), block.begin(), block.end());
                }
 
                // decrypt remaining bytes
                block.assign(p, p + input_length);
                dec->finish(block);
-               buffer_insert(plaintext, 0 + offset, block);
+               plaintext.insert(plaintext.end(), block.begin(), block.end());
 
                result.test_eq("decrypt update", plaintext, expected);
             }
@@ -313,7 +343,7 @@ class AEAD_Tests final : public Text_Based_Test {
          }
 
          // test decryption with modified ciphertext
-         const std::vector<uint8_t> mutated_input = mutate_vec(input, true);
+         const std::vector<uint8_t> mutated_input = mutate_vec(input, rng, true);
          buf.assign(mutated_input.begin(), mutated_input.end());
 
          dec->reset();
@@ -333,7 +363,7 @@ class AEAD_Tests final : public Text_Based_Test {
          // test decryption with modified nonce
          if(!nonce.empty()) {
             buf.assign(input.begin(), input.end());
-            std::vector<uint8_t> bad_nonce = mutate_vec(nonce);
+            std::vector<uint8_t> bad_nonce = mutate_vec(nonce, rng);
 
             dec->reset();
             dec->set_associated_data(ad);
@@ -350,7 +380,7 @@ class AEAD_Tests final : public Text_Based_Test {
          }
 
          // test decryption with modified associated_data
-         const std::vector<uint8_t> bad_ad = mutate_vec(ad, true);
+         const std::vector<uint8_t> bad_ad = mutate_vec(ad, rng, true);
 
          dec->reset();
          dec->set_associated_data(bad_ad);
@@ -372,11 +402,11 @@ class AEAD_Tests final : public Text_Based_Test {
          dec->clear();
          result.test_eq("key is not set", dec->has_keying_material(), false);
 
-         result.test_throws("Unkeyed object throws for decrypt", [&]() { dec->finish(buf); });
+         result.test_throws<Botan::Invalid_State>("Unkeyed object throws for decrypt", [&]() { dec->finish(buf); });
 
          if(dec->associated_data_requires_key()) {
-            result.test_throws("Unkeyed object throws for set AD",
-                               [&]() { dec->set_associated_data(ad.data(), ad.size()); });
+            result.test_throws<Botan::Invalid_State>("Unkeyed object throws for set AD",
+                                                     [&]() { dec->set_associated_data(ad.data(), ad.size()); });
          }
 
          return result;
@@ -426,10 +456,11 @@ class AEAD_Tests final : public Text_Based_Test {
                         enc->ideal_granularity() % enc->update_granularity() == 0);
 
          // test enc
-         result.merge(test_enc(key, nonce, input, expected, ad, algo));
+         result.merge(test_enc(key, nonce, input, expected, ad, algo, this->rng()));
 
          // test dec
-         result.merge(test_dec(key, nonce, expected, input, ad, algo));
+         // NOLINTNEXTLINE(*-suspicious-call-argument) Yes we are swapping ptext and ctext arguments here
+         result.merge(test_dec(key, nonce, expected, input, ad, algo, this->rng()));
 
          return result;
       }
