@@ -384,13 +384,14 @@ class BigInt_Div_Test final : public Text_Based_Test {
             result.test_eq("bw ok", Botan::BigInt::from_word(bw), b);
 
             Botan::BigInt ct_q;
-            Botan::word ct_r;
+            Botan::word ct_r = 0;
             Botan::ct_divide_word(a, bw, ct_q, ct_r);
             result.test_eq("ct_divide_word q", ct_q, c);
             result.test_eq("ct_divide_word r", ct_q * b + ct_r, a);
          }
 
-         Botan::BigInt ct_q, ct_r;
+         Botan::BigInt ct_q;
+         Botan::BigInt ct_r;
          Botan::ct_divide(a, b, ct_q, ct_r);
          result.test_eq("ct_divide q", ct_q, c);
          result.test_eq("ct_divide r", ct_q * b + ct_r, a);
@@ -474,12 +475,13 @@ class BigInt_Mod_Test final : public Text_Based_Test {
             result.test_eq("a % b (as word)", a % b_word, expected);
 
             Botan::BigInt ct_q;
-            Botan::word ct_r;
+            Botan::word ct_r = 0;
             Botan::ct_divide_word(a, b.word_at(0), ct_q, ct_r);
             result.test_eq("ct_divide_u8 r", ct_r, expected);
          }
 
-         Botan::BigInt ct_q, ct_r;
+         Botan::BigInt ct_q;
+         Botan::BigInt ct_r;
          Botan::ct_divide(a, b, ct_q, ct_r);
          result.test_eq("ct_divide r", ct_r, expected);
 
@@ -644,7 +646,7 @@ Test::Result test_const_time_left_shift() {
 
    Botan::BigInt a = Botan::BigInt::with_capacity(bits / sizeof(Botan::word));
    for(size_t i = 0; i < bits; ++i) {
-      if(rng->next_byte() & 1) {
+      if(rng->next_byte() % 2 == 1) {
          a.set_bit(i);
       }
    }
@@ -915,7 +917,8 @@ class DSA_ParamGen_Test final : public Text_Based_Test {
          Test::Result result("DSA Parameter Generation");
 
          try {
-            Botan::BigInt gen_P, gen_Q;
+            Botan::BigInt gen_P;
+            Botan::BigInt gen_Q;
             if(Botan::generate_dsa_primes(this->rng(), gen_P, gen_Q, p_bits, q_bits, seed, offset)) {
                result.test_eq("P", gen_P, exp_P);
                result.test_eq("Q", gen_Q, exp_Q);
