@@ -282,8 +282,11 @@ class Utility_Function_Tests final : public Test {
             result.test_is_eq<uint8_t>(out[7], 0xAB);
          }
 
-         std::array<uint8_t, 8> outarr;
-         uint16_t i0, i1, i2, i3;
+         std::array<uint8_t, 8> outarr{};
+         uint16_t i0 = 0;
+         uint16_t i1 = 0;
+         uint16_t i2 = 0;
+         uint16_t i3 = 0;
          Botan::store_be(in64, outarr);
 
          Botan::load_be(outarr, i0, i1, i2, i3);
@@ -396,7 +399,7 @@ class Utility_Function_Tests final : public Test {
          result.test_is_eq(Botan::store_be<std::vector<uint8_t>>(in16_vector), Botan::hex_decode("0A0B0C0D"));
          result.test_is_eq(Botan::store_le<std::vector<uint8_t>>(in16_vector), Botan::hex_decode("0B0A0D0C"));
 
-         std::array<uint8_t, 4> out_array;
+         std::array<uint8_t, 4> out_array{};
          Botan::store_be(out_array, in16_array);
          result.test_is_eq(out_array, std::array<uint8_t, 4>{0x0A, 0x0B, 0x0C, 0x0D});
          Botan::store_le(out_array, in16_array);
@@ -487,14 +490,14 @@ class Utility_Function_Tests final : public Test {
 
       template <std::unsigned_integral T>
       static decltype(auto) fb_store_be(const T in) {
-         std::array<uint8_t, sizeof(T)> out;
+         std::array<uint8_t, sizeof(T)> out{};
          Botan::detail::fallback_store_any<std::endian::big, T>(in, out);
          return out;
       }
 
       template <std::unsigned_integral T>
       static decltype(auto) fb_store_le(const T in) {
-         std::array<uint8_t, sizeof(T)> out;
+         std::array<uint8_t, sizeof(T)> out{};
          Botan::detail::fallback_store_any<std::endian::little, T>(in, out);
          return out;
       }
@@ -1112,6 +1115,39 @@ class Hostname_Tests final : public Text_Based_Test {
 
 BOTAN_REGISTER_TEST("utils", "hostname", Hostname_Tests);
 
+class DNS_Check_Tests final : public Text_Based_Test {
+   public:
+      DNS_Check_Tests() : Text_Based_Test("utils/dns.vec", "DNS") {}
+
+      Test::Result run_one_test(const std::string& type, const VarMap& vars) override {
+         Test::Result result("DNS name validation");
+
+         const std::string name = vars.get_req_str("DNS");
+         const bool valid = (type == "Invalid") ? false : true;
+
+         try {
+            const auto canonicalized = Botan::check_and_canonicalize_dns_name(name);
+            BOTAN_UNUSED(canonicalized);
+
+            if(valid) {
+               result.test_success("Accepted valid name");
+            } else {
+               result.test_failure("Accepted invalid name");
+            }
+         } catch(Botan::Decoding_Error&) {
+            if(valid) {
+               result.test_failure("Rejected valid name");
+            } else {
+               result.test_success("Rejected invalid name");
+            }
+         }
+
+         return result;
+      }
+};
+
+BOTAN_REGISTER_TEST("utils", "dns_check", DNS_Check_Tests);
+
 class IPv4_Parsing_Tests final : public Text_Based_Test {
    public:
       IPv4_Parsing_Tests() : Text_Based_Test("utils/ipv4.vec", "IPv4") {}
@@ -1166,12 +1202,12 @@ class ReadKV_Tests final : public Text_Based_Test {
          }
 
          std::string substr;
-         for(auto i = str.begin(); i != str.end(); ++i) {
-            if(*i == '|') {
+         for(char c : str) {
+            if(c == '|') {
                elems.push_back(substr);
                substr.clear();
             } else {
-               substr += *i;
+               substr += c;
             }
          }
 
@@ -1215,6 +1251,7 @@ class CPUID_Tests final : public Test {
          for(size_t b = 0; b != 32; ++b) {
             try {
                const auto bit = static_cast<uint32_t>(1) << b;
+               // NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
                const auto feat = Botan::CPUID::Feature(static_cast<Botan::CPUID::Feature::Bit>(bit));
 
                const std::string feat_str = feat.to_string();
@@ -1232,7 +1269,7 @@ class CPUID_Tests final : public Test {
             }
          }
 
-   #if defined(BOTAN_TARGET_CPU_IS_X86_FAMILY)
+   #if defined(BOTAN_TARGET_ARCH_IS_X86_FAMILY)
 
          const auto bit = Botan::CPUID::Feature::SSE2;
 
@@ -1248,6 +1285,8 @@ class CPUID_Tests final : public Test {
             result.test_eq(
                "After reinitializing, CPUID::has for SSE2 returns true again", Botan::CPUID::has(bit), true);
          }
+   #else
+         BOTAN_UNUSED(cpuid_string);
    #endif
 
          return {result};
@@ -1294,9 +1333,7 @@ class UUID_Tests : public Test {
                explicit AllSame_RNG(uint8_t b) : m_val(b) {}
 
                void fill_bytes_with_input(std::span<uint8_t> output, std::span<const uint8_t> /* ignored */) override {
-                  for(auto& byte : output) {
-                     byte = m_val;
-                  }
+                  std::fill(output.begin(), output.end(), m_val);
                }
 
                std::string name() const override { return "zeros"; }
