@@ -18,6 +18,10 @@
    #include <botan/x509path.h>
    #include <botan/x509self.h>
    #include <botan/internal/calendar.h>
+
+   #if defined(BOTAN_HAS_ECC_GROUP)
+      #include <botan/ec_group.h>
+   #endif
 #endif
 
 namespace Botan_Tests {
@@ -104,7 +108,12 @@ std::unique_ptr<Botan::Private_Key> make_a_private_key(const std::string& algo,
          return "1024";
       }
       if(algo == "GOST-34.10") {
-         return "gost_256A";
+   #if defined(BOTAN_HAS_ECC_GROUP)
+         if(Botan::EC_Group::supports_named_group("gost_256A")) {
+            return "gost_256A";
+         }
+   #endif
+         return "secp256r1";
       }
       if(algo == "ECKCDSA" || algo == "ECGDSA") {
          return "brainpool256r1";
@@ -118,7 +127,11 @@ std::unique_ptr<Botan::Private_Key> make_a_private_key(const std::string& algo,
       return "";  // default "" means choose acceptable algo-specific params
    }();
 
-   return Botan::create_private_key(algo, rng, params);
+   try {
+      return Botan::create_private_key(algo, rng, params);
+   } catch(Botan::Not_Implemented&) {
+      return {};
+   }
 }
 
 Test::Result test_cert_status_strings() {
@@ -160,6 +173,8 @@ Test::Result test_cert_status_strings() {
       Botan::Certificate_Status_Code::CERT_CHAIN_TOO_LONG,
       Botan::Certificate_Status_Code::CA_CERT_NOT_FOR_CERT_ISSUER,
       Botan::Certificate_Status_Code::NAME_CONSTRAINT_ERROR,
+      Botan::Certificate_Status_Code::IPADDR_BLOCKS_ERROR,
+      Botan::Certificate_Status_Code::AS_BLOCKS_ERROR,
       Botan::Certificate_Status_Code::CA_CERT_NOT_FOR_CRL_ISSUER,
       Botan::Certificate_Status_Code::OCSP_CERT_NOT_LISTED,
       Botan::Certificate_Status_Code::OCSP_BAD_STATUS,
@@ -405,6 +420,78 @@ Test::Result test_x509_dates() {
    return result;
 }
 
+Test::Result test_x509_encode_authority_info_access_extension() {
+   Test::Result result("X509 with encoded PKIX.AuthorityInformationAccess extension");
+
+   #if defined(BOTAN_HAS_RSA)
+   auto rng = Test::new_rng(__func__);
+
+   const std::string sig_algo{"RSA"};
+   const std::string hash_fn{"SHA-256"};
+   const std::string padding_method{"PKCS1v15(SHA-256)"};
+
+   // CA Issuer information
+   const std::vector<std::string> ca_issuers = {
+      "http://www.d-trust.net/cgi-bin/Bdrive_Test_CA_1-2_2017.crt",
+      "ldap://directory.d-trust.net/CN=Bdrive%20Test%20CA%201-2%202017,O=Bundesdruckerei%20GmbH,C=DE?cACertificate?base?"};
+
+   // OCSP
+   const std::string_view ocsp_uri{"http://staging.ocsp.d-trust.net"};
+
+   // create a CA
+   auto ca_key = make_a_private_key(sig_algo, *rng);
+   result.require("CA key", ca_key != nullptr);
+   const auto ca_cert = Botan::X509::create_self_signed_cert(ca_opts(), *ca_key, hash_fn, *rng);
+   Botan::X509_CA ca(ca_cert, *ca_key, hash_fn, padding_method, *rng);
+
+   // create a certificate with only caIssuer information
+   auto key = make_a_private_key(sig_algo, *rng);
+
+   Botan::X509_Cert_Options opts1 = req_opts1(sig_algo);
+   opts1.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>("", ca_issuers));
+
+   Botan::PKCS10_Request req = Botan::X509::create_cert_req(opts1, *key, hash_fn, *rng);
+
+   Botan::X509_Certificate cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
+
+   if(!result.test_eq("number of ca_issuers URIs", cert.ca_issuers().size(), 2)) {
+      return result;
+   }
+
+   for(const auto& ca_issuer : cert.ca_issuers()) {
+      result.confirm("CA issuer URI present in certificate",
+                     std::ranges::find(ca_issuers, ca_issuer) != ca_issuers.end());
+   }
+
+   result.confirm("no OCSP url available", cert.ocsp_responder().empty());
+
+   // create a certificate with only OCSP URI information
+   Botan::X509_Cert_Options opts2 = req_opts1(sig_algo);
+   opts2.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>(ocsp_uri));
+
+   req = Botan::X509::create_cert_req(opts2, *key, hash_fn, *rng);
+
+   cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
+
+   result.confirm("OCSP URI available", !cert.ocsp_responder().empty());
+   result.confirm("no CA Issuer URI available", cert.ca_issuers().empty());
+   result.test_eq("OCSP responder URI matches", cert.ocsp_responder(), std::string(ocsp_uri));
+
+   // create a certificate with OCSP URI and CA Issuer information
+   Botan::X509_Cert_Options opts3 = req_opts1(sig_algo);
+   opts3.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>(ocsp_uri, ca_issuers));
+
+   req = Botan::X509::create_cert_req(opts3, *key, hash_fn, *rng);
+
+   cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
+
+   result.confirm("OCSP URI available", !cert.ocsp_responder().empty());
+   result.confirm("CA Issuer URI available", !cert.ca_issuers().empty());
+   #endif
+
+   return result;
+}
+
    #if defined(BOTAN_TARGET_OS_HAS_FILESYSTEM)
 
 Test::Result test_crl_dn_name() {
@@ -570,7 +657,7 @@ Test::Result test_x509_authority_info_access_extension() {
    const auto ca_issuers = aia_cert.ca_issuers();
 
    result.test_eq("number of ca_issuers URLs", ca_issuers.size(), 1);
-   if(result.tests_failed()) {
+   if(result.tests_failed() > 0) {
       return result;
    }
 
@@ -584,7 +671,7 @@ Test::Result test_x509_authority_info_access_extension() {
    const auto ca_issuers2 = aia_cert_2ca.ca_issuers();
 
    result.test_eq("number of ca_issuers URLs", ca_issuers2.size(), 2);
-   if(result.tests_failed()) {
+   if(result.tests_failed() > 0) {
       return result;
    }
 
@@ -599,78 +686,6 @@ Test::Result test_x509_authority_info_access_extension() {
    return result;
 }
 
-Test::Result test_x509_encode_authority_info_access_extension() {
-   Test::Result result("X509 with encoded PKIX.AuthorityInformationAccess extension");
-
-      #if defined(BOTAN_HAS_RSA)
-   auto rng = Test::new_rng(__func__);
-
-   const std::string sig_algo{"RSA"};
-   const std::string hash_fn{"SHA-256"};
-   const std::string padding_method{"EMSA3(SHA-256)"};
-
-   // CA Issuer information
-   const std::vector<std::string> ca_issuers = {
-      "http://www.d-trust.net/cgi-bin/Bdrive_Test_CA_1-2_2017.crt",
-      "ldap://directory.d-trust.net/CN=Bdrive%20Test%20CA%201-2%202017,O=Bundesdruckerei%20GmbH,C=DE?cACertificate?base?"};
-
-   // OCSP
-   const std::string_view ocsp_uri{"http://staging.ocsp.d-trust.net"};
-
-   // create a CA
-   auto ca_key = make_a_private_key(sig_algo, *rng);
-   result.require("CA key", ca_key != nullptr);
-   const auto ca_cert = Botan::X509::create_self_signed_cert(ca_opts(), *ca_key, hash_fn, *rng);
-   Botan::X509_CA ca(ca_cert, *ca_key, hash_fn, padding_method, *rng);
-
-   // create a certificate with only caIssuer information
-   auto key = make_a_private_key(sig_algo, *rng);
-
-   Botan::X509_Cert_Options opts1 = req_opts1(sig_algo);
-   opts1.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>("", ca_issuers));
-
-   Botan::PKCS10_Request req = Botan::X509::create_cert_req(opts1, *key, hash_fn, *rng);
-
-   Botan::X509_Certificate cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
-
-   if(!result.test_eq("number of ca_issuers URIs", cert.ca_issuers().size(), 2)) {
-      return result;
-   }
-
-   for(const auto& ca_issuer : cert.ca_issuers()) {
-      result.confirm("CA issuer URI present in certificate",
-                     std::ranges::find(ca_issuers, ca_issuer) != ca_issuers.end());
-   }
-
-   result.confirm("no OCSP url available", cert.ocsp_responder().empty());
-
-   // create a certificate with only OCSP URI information
-   Botan::X509_Cert_Options opts2 = req_opts1(sig_algo);
-   opts2.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>(ocsp_uri));
-
-   req = Botan::X509::create_cert_req(opts2, *key, hash_fn, *rng);
-
-   cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
-
-   result.confirm("OCSP URI available", !cert.ocsp_responder().empty());
-   result.confirm("no CA Issuer URI available", cert.ca_issuers().empty());
-   result.test_eq("OCSP responder URI matches", cert.ocsp_responder(), std::string(ocsp_uri));
-
-   // create a certificate with OCSP URI and CA Issuer information
-   Botan::X509_Cert_Options opts3 = req_opts1(sig_algo);
-   opts3.extensions.add(std::make_unique<Botan::Cert_Extension::Authority_Information_Access>(ocsp_uri, ca_issuers));
-
-   req = Botan::X509::create_cert_req(opts3, *key, hash_fn, *rng);
-
-   cert = ca.sign_request(req, *rng, from_date(-1, 01, 01), from_date(2, 01, 01));
-
-   result.confirm("OCSP URI available", !cert.ocsp_responder().empty());
-   result.confirm("CA Issuer URI available", !cert.ca_issuers().empty());
-      #endif
-
-   return result;
-}
-
 Test::Result test_parse_rsa_pss_cert() {
    Test::Result result("X509 RSA-PSS certificate");
 
@@ -691,17 +706,19 @@ Test::Result test_verify_gost2012_cert() {
 
       #if defined(BOTAN_HAS_GOST_34_10_2012) && defined(BOTAN_HAS_STREEBOG)
    try {
-      Botan::X509_Certificate root_cert(Test::data_file("x509/gost/gost_root.pem"));
-      Botan::X509_Certificate root_int(Test::data_file("x509/gost/gost_int.pem"));
+      if(Botan::EC_Group::supports_named_group("gost_256A")) {
+         Botan::X509_Certificate root_cert(Test::data_file("x509/gost/gost_root.pem"));
+         Botan::X509_Certificate root_int(Test::data_file("x509/gost/gost_int.pem"));
 
-      Botan::Certificate_Store_In_Memory trusted;
-      trusted.add_certificate(root_cert);
+         Botan::Certificate_Store_In_Memory trusted;
+         trusted.add_certificate(root_cert);
 
-      const Botan::Path_Validation_Restrictions restrictions(false, 128, false, {"Streebog-256"});
-      const Botan::Path_Validation_Result validation_result =
-         Botan::x509_path_validate(root_int, restrictions, trusted);
+         const Botan::Path_Validation_Restrictions restrictions(false, 128, false, {"Streebog-256"});
+         const Botan::Path_Validation_Result validation_result =
+            Botan::x509_path_validate(root_int, restrictions, trusted);
 
-      result.confirm("GOST certificate validates", validation_result.successful_validation());
+         result.confirm("GOST certificate validates", validation_result.successful_validation());
+      }
    } catch(const Botan::Decoding_Error& e) {
       result.test_failure(e.what());
    }
@@ -710,14 +727,13 @@ Test::Result test_verify_gost2012_cert() {
    return result;
 }
 
-      /*
- * @brief checks the configurability of the EMSA4(RSA-PSS) signature scheme
+   /*
+ * @brief checks the configurability of the RSA-PSS signature scheme
  *
  * For the other algorithms than RSA, only one padding is supported right now.
  */
       #if defined(BOTAN_HAS_EMSA_PKCS1) && defined(BOTAN_HAS_EMSA_PSSR) && defined(BOTAN_HAS_RSA)
 Test::Result test_padding_config() {
-   // Throughout the test, some synonyms for EMSA4 are used, e.g. PSSR, EMSA-PSS
    Test::Result test_result("X509 Padding Config");
 
    auto rng = Test::new_rng(__func__);
@@ -725,93 +741,91 @@ Test::Result test_padding_config() {
    Botan::DataSource_Stream key_stream(Test::data_file("x509/misc/rsa_key.pem"));
    auto sk = Botan::PKCS8::load_key(key_stream);
 
-   // Create X509 CA certificate; EMSA3 is used for signing by default
+   // Create X509 CA certificate; PKCS1v15 is used for signing by default
    Botan::X509_Cert_Options opt("TESTCA");
    opt.CA_key();
 
    Botan::X509_Certificate ca_cert_def = Botan::X509::create_self_signed_cert(opt, (*sk), "SHA-512", *rng);
    test_result.test_eq("CA certificate signature algorithm (default)",
                        ca_cert_def.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA3(SHA-512)");
+                       "RSA/PKCS1v15(SHA-512)");
 
    // Create X509 CA certificate; RSA-PSS is explicitly set
    opt.set_padding_scheme("PSSR");
    Botan::X509_Certificate ca_cert_exp = Botan::X509::create_self_signed_cert(opt, (*sk), "SHA-512", *rng);
    test_result.test_eq("CA certificate signature algorithm (explicit)",
                        ca_cert_exp.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA4");
+                       "RSA/PSS");
 
-         #if defined(BOTAN_HAS_EMSA2)
+         #if defined(BOTAN_HAS_EMSA_X931)
    // Try to set a padding scheme that is not supported for signing with the given key type
-   opt.set_padding_scheme("EMSA2");
+   opt.set_padding_scheme("X9.31");
    try {
       Botan::X509_Certificate ca_cert_wrong = Botan::X509::create_self_signed_cert(opt, (*sk), "SHA-512", *rng);
-      test_result.test_failure("Could build CA cert with invalid encoding scheme EMSA1 for key type " +
+      test_result.test_failure("Could build CA cert with invalid encoding scheme X9.31 for key type " +
                                sk->algo_name());
    } catch(const Botan::Invalid_Argument& e) {
-      test_result.test_eq("Build CA certificate with invalid encoding scheme EMSA1 for key type " + sk->algo_name(),
+      test_result.test_eq("Build CA certificate with invalid encoding scheme X9.31 for key type " + sk->algo_name(),
                           e.what(),
-                          "Signatures using RSA/EMSA2(SHA-512) are not supported");
+                          "Signatures using RSA/X9.31(SHA-512) are not supported");
    }
          #endif
 
    test_result.test_eq("CA certificate signature algorithm (explicit)",
                        ca_cert_exp.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA4");
+                       "RSA/PSS");
 
    const Botan::X509_Time not_before = from_date(-1, 1, 1);
    const Botan::X509_Time not_after = from_date(2, 1, 2);
 
    // Prepare a signing request for the end certificate
    Botan::X509_Cert_Options req_opt("endpoint");
-   req_opt.set_padding_scheme("EMSA4(SHA-512,MGF1,64)");
+   req_opt.set_padding_scheme("PSS(SHA-512,MGF1,64)");
    Botan::PKCS10_Request end_req = Botan::X509::create_cert_req(req_opt, (*sk), "SHA-512", *rng);
-   test_result.test_eq("Certificate request signature algorithm",
-                       end_req.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA4");
+   test_result.test_eq(
+      "Certificate request signature algorithm", end_req.signature_algorithm().oid().to_formatted_string(), "RSA/PSS");
 
    // Create X509 CA object: will fail as the chosen hash functions differ
    try {
-      Botan::X509_CA ca_fail(ca_cert_exp, (*sk), "SHA-512", "EMSA4(SHA-256)", *rng);
+      Botan::X509_CA ca_fail(ca_cert_exp, (*sk), "SHA-512", "PSS(SHA-256)", *rng);
       test_result.test_failure("Configured conflicting hash functions for CA");
    } catch(const Botan::Invalid_Argument& e) {
       test_result.test_eq(
          "Configured conflicting hash functions for CA",
          e.what(),
-         "Specified hash function SHA-512 is incompatible with RSA chose hash function SHA-256 with user specified padding EMSA4(SHA-256)");
+         "Specified hash function SHA-512 is incompatible with RSA chose hash function SHA-256 with user specified padding PSS(SHA-256)");
    }
 
-   // Create X509 CA object: its signer will use the padding scheme from the CA certificate, i.e. EMSA3
+   // Create X509 CA object: its signer will use the padding scheme from the CA certificate, i.e. PKCS1v15
    Botan::X509_CA ca_def(ca_cert_def, (*sk), "SHA-512", *rng);
-   Botan::X509_Certificate end_cert_emsa3 = ca_def.sign_request(end_req, *rng, not_before, not_after);
+   Botan::X509_Certificate end_cert_pkcs1 = ca_def.sign_request(end_req, *rng, not_before, not_after);
    test_result.test_eq("End certificate signature algorithm",
-                       end_cert_emsa3.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA3(SHA-512)");
+                       end_cert_pkcs1.signature_algorithm().oid().to_formatted_string(),
+                       "RSA/PKCS1v15(SHA-512)");
 
    // Create X509 CA object: its signer will use the explicitly configured padding scheme, which is different from the CA certificate's scheme
-   Botan::X509_CA ca_diff(ca_cert_def, (*sk), "SHA-512", "EMSA-PSS", *rng);
-   Botan::X509_Certificate end_cert_diff_emsa4 = ca_diff.sign_request(end_req, *rng, not_before, not_after);
+   Botan::X509_CA ca_diff(ca_cert_def, (*sk), "SHA-512", "PSS", *rng);
+   Botan::X509_Certificate end_cert_diff_pss = ca_diff.sign_request(end_req, *rng, not_before, not_after);
    test_result.test_eq("End certificate signature algorithm",
-                       end_cert_diff_emsa4.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA4");
+                       end_cert_diff_pss.signature_algorithm().oid().to_formatted_string(),
+                       "RSA/PSS");
 
    // Create X509 CA object: its signer will use the explicitly configured padding scheme, which is identical to the CA certificate's scheme
-   Botan::X509_CA ca_exp(ca_cert_exp, (*sk), "SHA-512", "EMSA4(SHA-512,MGF1,64)", *rng);
-   Botan::X509_Certificate end_cert_emsa4 = ca_exp.sign_request(end_req, *rng, not_before, not_after);
-   test_result.test_eq("End certificate signature algorithm",
-                       end_cert_emsa4.signature_algorithm().oid().to_formatted_string(),
-                       "RSA/EMSA4");
+   Botan::X509_CA ca_exp(ca_cert_exp, (*sk), "SHA-512", "PSS(SHA-512,MGF1,64)", *rng);
+   Botan::X509_Certificate end_cert_pss = ca_exp.sign_request(end_req, *rng, not_before, not_after);
+   test_result.test_eq(
+      "End certificate signature algorithm", end_cert_pss.signature_algorithm().oid().to_formatted_string(), "RSA/PSS");
 
    // Check CRL signature algorithm
    Botan::X509_CRL crl = ca_exp.new_crl(*rng);
-   test_result.test_eq("CRL signature algorithm", crl.signature_algorithm().oid().to_formatted_string(), "RSA/EMSA4");
+   test_result.test_eq("CRL signature algorithm", crl.signature_algorithm().oid().to_formatted_string(), "RSA/PSS");
 
    // sanity check for verification, the heavy lifting is done in the other unit tests
    const Botan::Certificate_Store_In_Memory trusted(ca_exp.ca_certificate());
    const Botan::Path_Validation_Restrictions restrictions(false, 80);
    const Botan::Path_Validation_Result validation_result =
-      Botan::x509_path_validate(end_cert_emsa4, restrictions, trusted);
-   test_result.confirm("EMSA4-signed certificate validates", validation_result.successful_validation());
+      Botan::x509_path_validate(end_cert_pss, restrictions, trusted);
+   test_result.confirm("PSS signed certificate validates", validation_result.successful_validation());
 
    return test_result;
 }
@@ -898,7 +912,7 @@ Test::Result test_x509_cert(const Botan::Private_Key& ca_key,
    /* Create the CA object */
    Botan::X509_CA ca(ca_cert, ca_key, hash_fn, sig_padding, rng);
 
-   const BigInt user1_serial = 99;
+   const BigInt user1_serial(99);
 
    /* Sign the requests to create the certs */
    Botan::X509_Certificate user1_cert =
@@ -908,6 +922,7 @@ Test::Result test_x509_cert(const Botan::Private_Key& ca_key,
    result.test_eq("User1 serial matches expected", user1_cert.serial_number().at(0), size_t(99));
 
    Botan::X509_Certificate user2_cert = ca.sign_request(user2_req, rng, from_date(-1, 01, 01), from_date(2, 01, 01));
+   result.test_eq("extended key usage is set", user2_cert.has_ex_constraint("PKIX.EmailProtection"), true);
 
    Botan::X509_Certificate user3_cert = ca.sign_request(user3_req, rng, from_date(-1, 01, 01), from_date(2, 01, 01));
 
@@ -990,7 +1005,7 @@ Test::Result test_x509_cert(const Botan::Private_Key& ca_key,
 
    std::vector<Botan::CRL_Entry> revoked;
    revoked.push_back(Botan::CRL_Entry(user1_cert, Botan::CRL_Code::CessationOfOperation));
-   revoked.push_back(user2_cert);
+   revoked.push_back(Botan::CRL_Entry(user2_cert));
 
    const Botan::X509_CRL crl2 = ca.update_crl(crl1, revoked, rng);
 
@@ -1200,7 +1215,7 @@ Test::Result test_valid_constraints(const Botan::Private_Key& key, const std::st
       result.test_eq("usage acceptable", key_agreement_decipher_only.compatible_with(key), true);
       result.test_eq("crl sign not permitted", crl_sign.compatible_with(key), false);
       result.test_eq("sign", sign_everything.compatible_with(key), false);
-   } else if(pk_algo == "Kyber" || pk_algo == "FrodoKEM" || pk_algo == "ML-KEM") {
+   } else if(pk_algo == "Kyber" || pk_algo == "FrodoKEM" || pk_algo == "ML-KEM" || pk_algo == "ClassicMcEliece") {
       // KEMs can encrypt and agree
       result.test_eq("all constraints not permitted", all.compatible_with(key), false);
       result.test_eq("cert sign not permitted", ca.compatible_with(key), false);
@@ -1420,7 +1435,7 @@ Test::Result test_x509_extensions(const Botan::Private_Key& ca_key,
    }
 
    // check if CDPs are present in the self-signed cert
-   auto cert_cdps =
+   const auto* cert_cdps =
       self_signed_cert.v3_extensions().get_extension_object_as<Botan::Cert_Extension::CRL_Distribution_Points>();
 
    if(result.confirm("CRL Distribution Points extension present in self-signed certificate",
@@ -1521,6 +1536,8 @@ Test::Result test_hashes(const Botan::Private_Key& key, const std::string& hash_
    return result;
 }
 
+   #if defined(BOTAN_TARGET_OS_HAS_FILESYSTEM)
+
 Test::Result test_x509_tn_auth_list_extension_decode() {
    /* cert with TNAuthList extension data was generated by asn1parse cfg:
 
@@ -1545,9 +1562,9 @@ Test::Result test_x509_tn_auth_list_extension_decode() {
 
    using Botan::Cert_Extension::TNAuthList;
 
-   auto tn_auth_list = cert.v3_extensions().get_extension_object_as<TNAuthList>();
+   const auto* tn_auth_list = cert.v3_extensions().get_extension_object_as<TNAuthList>();
 
-   auto& tn_entries = tn_auth_list->entries();
+   const auto& tn_entries = tn_auth_list->entries();
 
    result.confirm("cert has TNAuthList extension", tn_auth_list != nullptr, true);
 
@@ -1562,7 +1579,7 @@ Test::Result test_x509_tn_auth_list_extension_decode() {
    result.test_eq("spc entry data", tn_entries[0].service_provider_code(), "1001");
 
    result.test_eq("range entry type", tn_entries[1].type() == TNAuthList::Entry::TelephoneNumberRange, true);
-   auto& range = tn_entries[1].telephone_number_range();
+   const auto& range = tn_entries[1].telephone_number_range();
    result.test_eq("range entries count", range.size(), 2);
    result.test_eq("range entry 0 start data", range[0].start.value(), "111");
    result.test_eq("range entry 0 count data", range[0].count, 128);
@@ -1576,9 +1593,11 @@ Test::Result test_x509_tn_auth_list_extension_decode() {
    return result;
 }
 
+   #endif
+
 std::vector<std::string> get_sig_paddings(const std::string& sig_algo, const std::string& hash) {
    if(sig_algo == "RSA") {
-      return {"EMSA3(" + hash + ")", "EMSA4(" + hash + ")"};
+      return {"PKCS1v15(" + hash + ")", "PSS(" + hash + ")"};
    } else if(sig_algo == "DSA" || sig_algo == "ECDSA" || sig_algo == "ECGDSA" || sig_algo == "ECKCDSA" ||
              sig_algo == "GOST-34.10") {
       return {hash};
@@ -1695,7 +1714,8 @@ class X509_Cert_Unit_Tests final : public Test {
          /*
          These are algos which cannot sign but can be included in certs
          */
-         const std::vector<std::string> enc_algos = {"DH", "ECDH", "ElGamal", "Kyber", "ML-KEM", "FrodoKEM"};
+         const std::vector<std::string> enc_algos = {
+            "DH", "ECDH", "ElGamal", "Kyber", "ML-KEM", "FrodoKEM", "ClassicMcEliece"};
 
          for(const std::string& algo : enc_algos) {
             auto key = make_a_private_key(algo, rng);
